"use strict";
angular.module('starter', ['ionic',
  'starter.controllers','starter.controllerrefer','starter.controllercalc','starter.controlFeedback','starter.controllerHistory',
  'starter.controllerpdf',
  'sync-service','history-service',
  'starter.controllerQuestions'])

.run(function($ionicPlatform,$rootScope,syncableDatabase) {
  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if (window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
  });

  /*
  must be outside $ionicPlatform.ready
  or value never set right, and handlers don't fire when changes.
  */
  document.addEventListener("deviceready", onDeviceReady );


  var networkSync= function _networkSync(){
    console.info('network sync triggered @ '+new Date());
    syncableDatabase.sync();//this is OK to call frequently. It will debounce itself
  };

  function stopNetworkSync(){
    clearInterval( $rootScope.networkSyncInterval )
  }
  function startNetworkSync(){
    networkSync();
  }

  function onDeviceReady() {
    window.hasFiredDeviceReady=true;

    var id = window.device?window.device.uuid:'test-uid';
    var p = ionic.Platform.platform();

    $rootScope.$apply(function() {
      $rootScope.id = id;
      $rootScope.p = p;
      $rootScope.online = navigator.onLine;
    });

    syncableDatabase.init(p,id);

    if( $rootScope.online ){
      startNetworkSync();
    }
    document.addEventListener("offline", function() {
        $rootScope.$apply(function() {//needs apply or doesn't update
            $rootScope.online = false;
        });
        stopNetworkSync();
    }, false);
    document.addEventListener("online", function() {
        $rootScope.$apply(function() {//needs apply or doesn't update
            $rootScope.online = true;
        });
        startNetworkSync();
    }, false);
  };

})

.config(function($stateProvider, $urlRouterProvider) {
  $stateProvider

  // setup an abstract state for the tabs
  .state('app', {
    url: "/app",
    abstract: true,
    templateUrl: "templates/tabs.html",
    controller: 'AppCtrl'
  })

  .state('app.home', {
    url: "/home",
    views: {
      'tab-home': {
        templateUrl: "templates/home.html"
      }
    }
  })

  .state('app.menu', {
    url: "/menu",
    views: {
      'tab-menu': {
        templateUrl: "templates/menu.html",
        controller: "HomeCtrl"
      }
    }
  })
  .state('app.more', {
    url: "/home/more",
    views: {
      'tab-home': {
        templateUrl: "templates/more.html",
        controller: "HomeCtrl"
      }
    }
  })

  .state('app.calc-type',{
    url:'/calc-type',
    views: {
      'tab-calc-type': {
        templateUrl: "templates/calc-type.html",
        controller: 'CalcTypeController'
      }
    }
  })

  .state('app.calc-urine',{
    url:'/calc-urine',
    views: {
      'tab-calc-urine': {
        templateUrl: "templates/calc-urine.html",
        controller: 'CalcUrineController'
      }
    }
  })

  .state('app.calc', {
    url: "/calc",
    views: {
      'tab-calc': {
        templateUrl: "templates/calc.html",
        controller: "CalcController"
      }
    },
    resolve:{
      wasLastResultTest : function(testService){return testService.wasLastResultTest();}
    }
  })
  .state('app.calc-further', {
    url: "/calc/further/:testResults_id",
    views: {
      'tab-calc-further': {
        templateUrl: "templates/calc-further.html",
        controller: 'CalcFurtherController'
      }
    }
  })
  .state('app.risk-asses', {
    url: "/calc/assess/:testResults_id",
    views: {
      'tab-calc-assess': {
        templateUrl: "templates/calc-risk-assess.html",
        controller: 'CalcRiskAccessController'
      }
    }
  })

  .state('app.hype', {
    url: "/calc/assess/hype/:type",
    params:{
      type:{value:'moderate'}
    },
    views: {
      'tab-calc-assess-hype': {
        templateUrl: "templates/risk/hype.html",
        controller: 'CalcHypeController'
      }
    }
  })
 .state('app.hype-waves', {
    url: "/calc/assess/hype/waves",
    views: {
      'tab-calc-assess-hype-wave': {
        templateUrl: "templates/risk/waveHype.html",
        controller: 'CalcWaveController'
      }
    }
 })

  .state('app.treatment', {
    url: "/calc/treatment/:testResults_id",
    views: {
      'tab-calc-treatment': {
        templateUrl: "templates/calc-risk-treatment.html",
        controller: 'CalcTreatmentController'
      }
    }
  })

  //https://github.com/angular-ui/ui-router/issues/108#issuecomment-54349456 - optional args
  .state('app.refer', {
    url: "/refer/:testResults_id/:score",
     params: {
          testResults_id:{
              value: '-1'
          },
          score:{
              value: '-1',
              squash: true
          } //The parameter's default value is omitted from the U
      },
    views: {
      'tab-refer': {
        templateUrl: "templates/refer.html",
        controller: 'ReferController'
      }
    },
    resolve:{
      wasLastResultTest : function(testService){return testService.wasLastResultTest();}
    }
  })
  .state('app.refer-safety', {
    url: "/refer/safety/:refer_id/:testResults_id",
    views: {
      'tab-safety': {
        templateUrl: "templates/refer-safety.html",
        controller: 'ReferSafetyController'
      }
    }
  })
  .state('app.refer-result', {
    url: "/refer/result/:referSafety_id/:safe",
    views: {
      'tab-safety-result': {
        templateUrl: "templates/refer-result.html",
        controller: 'ReferResultController'
      }
    }
  })

  .state('app.links', {
    url: "/links",
    views: {
      'tab-links': {
        templateUrl: "templates/links.html",
        controller: 'LinksCtrl'
      }
    }
  })
  .state('app.summary', {
    url: "/summary",
    views: {
      'tab-summary': {
        templateUrl: "templates/summary.html",
        controller: 'SummaryCtrl'
      }
    }
  })
  .state('app.about', {
    url: "/about",
    views: {
      'tab-about': {
        templateUrl: "templates/about.html"
      }
    }
  })
  .state('app.feedback', {
    url: "/feedback",
    views: {
      'tab-feedback': {
        templateUrl: "templates/feedback.html",
        controller: 'FeedbackController'
      }
    }
  })

  .state('app.history', {
    url: "/history",
    views: {
      'tab-history': {
        templateUrl: "templates/history.html",
        controller: 'HistoryController'
      }
    },
    resolve:{
      wasLastResultTest : function(testService){return testService.wasLastResultTest();}
    }
  })
  .state('app.history-detail', {
    url: "/history/:table/:detail",
    views: {
      'tab-history-detail': {
        templateUrl: "templates/history-detail.html",
        controller: 'HistoryDetailController'
      }
    }
  })

  .state('app.discharge', {
    url:'/discharge',
    views:{
      'tab-discharge':{
        templateUrl:'templates/discharge.html'
      }
    }
  })

  .state('app.units', {
    url:'/units',
    views:{
      'tab-units':{
        templateUrl:'templates/units.html',
        controller:'UnitsCtrl'
      }
    }
  })

  .state('app.pdf', {
    url: "/pdf/:pdfIndex/:pageIndex",
    views: {
      'tab-pdf': {
        templateUrl: "templates/pdf.html",
        controller: 'PdfController'
      }
    }
  })

  ;

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/app/home');
});
