"use strict";
angular.module('starter.controllercalc',[
  'sync-service','email-ask.service',"email-service",'register-service','calcState-service',
  'test-preferences','test-ask.service',
  'urineOutput-preferences',
  'starter.controllerQuestions'
])

.controller('CalcTreatmentController', function($scope,$state,$ionicPopup, syncableDatabase,QuestionsController){
  $scope.showHelp  = function(evt){
    var test=_testByTestId(evt.target.id);
    var template = '<ion-popover-view><ion-header-bar><h3>'+test.label+
    '</h3></ion-header-bar><ion-content><ion-scroll><p>'+test.help+
    '</p><p><a type="submit" class="button button-wide button-stable button-small" ng-click="hideHelp()">close</a></p><ion-scroll>/ion-content></ion-popover-view>';

    $scope.ionicPopup = $ionicPopup.alert({
      title:test.label,
      template:'<p>'+test.help+'</p>',
      okText:'close',
      okType: 'button-balanced'
    });
  }

  $scope.continueToReferral=function(){
    syncableDatabase.db().transaction(function(tx) {
      tx.executeSql("select score from testResults where id=?",[ $state.params.testResults_id ],function(tx,results){
        var params=[new Date().getTime(),$state.params.testResults_id ];

        var sql='insert into treatment (date,testResults_id';

        for (var s in $scope.steps ){
          for(var i=0;i<$scope.steps[s].tests.length;i++){
            var t=$scope.steps[s].tests[i];
            sql+=','+t.id;
            params=params.concat( t.value );
          }
        }

        sql+=') values (';

        var valueSql='';//one '?,' for every entry in params
        for(var i=0;i < params.length;i++){
          if( i == params.length-1 ){
            sql+='?';
          }else{
            sql+='?,';
          }
        }

        sql+=')';

        tx.executeSql(
            sql,params,
            function(){ syncableDatabase.sync(); },
            function(t,e){console.error('continueToReferral : failed to save');console.dir(e);}
        );

        $state.go( "app.refer",{
              testResults_id:$state.params.testResults_id,
              score :  results.rows.length==1?results.rows.item(0).score:-1
        });
      });
    });
  };

   $scope.steps = QuestionsController.getQuestions('stepsTreat');

   function _testByTestId(id){
    for (var s in $scope.steps ){
      for(var i=0;i<$scope.steps[s].tests.length;i++){
        if( $scope.steps[s].tests[i].id == id ){
          return $scope.steps[s].tests[i];
        }
      }
    }
   }
})

.controller('CalcRiskAccessController', function($scope,$ionicModal,$state,$ionicScrollDelegate, syncableDatabase){
  $scope.risk={};
  $scope.risk.pot='';
  $scope.risk.ph='';

  $scope.risk.confusion='';
  $scope.risk.flap='';
  $scope.risk.rub='';
  $scope.risk.oedema='';

  $ionicModal.fromTemplateUrl('templates/risk/acidosis.html', {
        scope: $scope
  }).then(function(modal) {
    $scope._modalAcidosis = modal;
  });
  $ionicModal.fromTemplateUrl('templates/risk/uraemia.html', {
        scope: $scope
  }).then(function(modal) {
    $scope._modalUraemia = modal;
  });
  $ionicModal.fromTemplateUrl('templates/risk/oedema.html', {
        scope: $scope
  }).then(function(modal) {
    $scope._modalOedema = modal;
  });

  $scope.$on('$destroy', function() {
    $scope._modalAcidosis.remove();
    $scope._modalUraemia.remove();
    $scope._modalOedema.remove();
  });

  $scope.showPot=function(e){
    if( $scope.risk.pot == null ||
        $scope.risk.pot == '' ){
      return;
    }
    if(
      $scope.risk.pot >= 5.5 &&
      $scope.risk.pot < 6.1
     ){
      $state.go('app.hype',{type:'moderate'});
    }else if(
      $scope.risk.pot >= 6.1
    ){
      $state.go('app.hype',{type:'severe'});
    }
  }
  $scope.showPh=function(e){
    if( $scope.risk.ph == null ||
        $scope.risk.ph == '' ){
      return;
    }
    if( $scope.risk.ph < 7.35 ){
      $scope.modal=$scope._modalAcidosis;
      $scope.modal.show(e);
    }
  }

  $scope.showUraemia=function(){
    $scope.modal=$scope._modalUraemia;
    $scope.modal.show();
  }
  $scope.showOedema=function(){
    $scope.modal=$scope._modalOedema;
    $scope.modal.show();
  }

  $scope.confusionClick=function(evt){
    if( evt.target.nodeName == 'DIV' ){
      showUraemia();
      return;
    }
    if( evt.target.nodeName == 'INPUT' ){
      return;
    }
    $scope.risk.confusion=!$scope.risk.confusion;
    hackToFixHeightBug();
  }
  $scope.flapClick=function(evt){
    if( evt.target.nodeName == 'DIV' ){
      showUraemia();
      return;
    }
    if( evt.target.nodeName == 'INPUT' ){
      return;
    }
    $scope.risk.flap=!$scope.risk.flap;
    hackToFixHeightBug();
  }
  $scope.rubClick=function(evt){
    if( evt.target.nodeName == 'DIV' ){
      showUraemia();
      return;
    }
    if( evt.target.nodeName == 'INPUT' ){
      return;
    }
    $scope.risk.rub=!$scope.risk.rub;
    hackToFixHeightBug();
  }
  $scope.oedemaClick=function(evt){
    if( evt.target.nodeName == 'DIV' ){
      showOedema();
      return;
    }
    if( evt.target.nodeName == 'INPUT' ){
      return;
    }
    $scope.risk.oedema=!$scope.risk.oedema;
    hackToFixHeightBug();
  }

  $scope.continueTreatment=function(){
    syncableDatabase.db().transaction(function(tx) {
      var params=[new Date().getTime(),$state.params.testResults_id,
        $scope.risk.pot,
        $scope.risk.ph,

        $scope.risk.confusion,
        $scope.risk.flap,
        $scope.risk.rub,
        $scope.risk.oedema,
      ];

      tx.executeSql('insert into riskAssesment (date,testResults_id,pot,ph,confusion,flap,rub,oedema) values (?,?,?,?,?,?,?,?)',params);
    },
    function(e){ console.error('failed to save');console.dir(e); },
    function(){ syncableDatabase.sync(); }
    );

    $state.go( "app.treatment",{
          testResults_id:$state.params.testResults_id
    });
  }

  $scope.$modalCancel = function(){
    $scope.modal.hide();
  }

  function hackToFixHeightBug(){
    // https://github.com/driftyco/ionic/issues/3591
    var scr=$ionicScrollDelegate.$getByHandle('calcRiskScrollHandle');
    scr.scrollTo(0,scr.getScrollPosition().top,true);
  }

})

.controller('CalcHypeController', function($scope,$state,geoService){
  $scope.type=$state.params.type;
  $scope.region = geoService.getCurrentRegion().name
  $scope.getRegion = function() {
    if ($scope.region == 'Greater Manchester, Lancashire and South Cumbria Strategic Clinical Network') {
      return true
    } else {
      return false
    }
  }

})
.controller('CalcWaveController', function($scope,$window,$ionicScrollDelegate){
  var w = $window;
  var offset = ( (document.getElementsByTagName('body')[0].className.indexOf('ios')>-1) ?330:320);

  var initZoom=1-(window.innerWidth/764);

  $scope.$on('$ionicView.enter',function(){
    $ionicScrollDelegate.$getByHandle('wavesScroller').zoomBy(initZoom);
     console.log('initZoom '+initZoom);
  });

  var st={
                'height': (w.innerHeight - offset ) + 'px',
                'width': (w.innerWidth - 20) + 'px'
            };
  $scope.style = function () {
    return st;
  };
})

.controller('CalcFurtherController', function($rootScope,$scope, $state, syncableDatabase, createEmailDialog,$ionicPopover,emailService,$ionicLoading,$ionicPopup,QuestionsController) {

  $scope.$on('$ionicView.leave', function(){
    //reset risks
    for( var i=0;i < $scope.factors.length;i++ ){
      for( var j=0;j < $scope.factors[i].risks.length;j++ ){
        $scope.factors[i].risks[j].value = undefined;
      }
    }
  });

  $scope.allComplete = function(){
    for( var i=0;i < $scope.factors.length;i++ ){
      for( var j=0;j < $scope.factors[i].risks.length;j++ ){
        if(
          $scope.factors[i].risks[j].valueYes == null &&
          $scope.factors[i].risks[j].valueNo  == null
        ){
          return false;
        }
      }
    }
    return true;
  }

  $scope.back = function(){
    $scope._saveFurther(false);
    if( $state.params.testResults_id != '' ){
      $state.go("app.calc-type");
    }else{
      $state.go("app.home");
    }
  };
  $scope.email = function(evt){

    syncableDatabase.executeSql('select * from testResults where id=?',[$state.params.testResults_id])
    .then(function(results){

      var id    = results.rows.length == 1 ? results.rows.item(0).id : '' ;
      var score = results.rows.length == 1 ? results.rows.item(0)    : {} ;

      //get email address
      createEmailDialog(evt,{
        success:  function(email){
          console.log('Simple modal closed '+email);

          $ionicLoading.show({
            template: 'Sending email...',
            cssClass: 'review'
          });

          var risksAsBool=[];
          for( var i=0;i < $scope.factors.length;i++ ){
            for( var j=0;j < $scope.factors[i].risks.length;j++ ){
              risksAsBool.push({
                id    :  $scope.factors[i].risks[j].id,
                value : (
                  typeof $scope.factors[i].risks[j].valueYes == 'boolean' &&
                         $scope.factors[i].risks[j].valueYes == true
                )
              });
            }
          }
          emailService.sendEmail('furtherAdvice_2',email,{
            score : score,
            risks : risksAsBool
          })
          .then(function(data,status){
            $ionicLoading.hide();
            if(data.ok == true ){
              // - set email done flag
                $scope._saveFurther(id,true,email,true);
                $ionicPopup.alert({
                  title: 'Email sent',
                  template: 'The email will be with you shortly',
                  cssClass: 'further ok',
                  okType: 'button-balanced'
                });
              }else{
                console.error("remote error "+data.message);
                //as error()
                $ionicPopup.alert({
                  title: 'Email sending delay',
                  template: 'The email will be with you after a short delay',
                  cssClass: 'further delay',
                  okType: 'button-balanced'
                });
                $scope._saveFurther(id,true,email,false);
              }
          })
          .catch(function(data,status){
            $ionicLoading.hide();
            console.error("failed to call "+data);
            $ionicPopup.alert({
              title: 'Could not find server' ,
              template: 'The email will be sent next time you go online',
              cssClass: 'further warn',
              okType: 'button-balanced'
            });
            $scope._saveFurther(id,true,email,false);
          });

          $state.go("app.home");
        }
      });
    });
  };

  $scope.itemClick=function(evt,i,j){

    if(  evt.target.nodeName == 'INPUT' ){// only the check box itself
      //console.log('@'+i+' y:'+$scope.risks[i].valueYes+' n:'+$scope.risks[i].valueNo);

      //first click, honour what tapped on
      if( $scope.factors[i].risks[j].valueYes == null &&
          $scope.factors[i].risks[j].valueNo  == null
      ){
        if( evt.target.getAttribute('ng-model').indexOf('valueNo') > -1 ){
          $scope.factors[i].risks[j].valueYes = false;
        }else{
          $scope.factors[i].risks[j].valueNo  = false;
        }
        return;
      }

      //which of yes and no was just clicked and changed value ? Toggle the other
      if( evt.target.getAttribute('ng-model').indexOf('valueNo') > -1 ){
        $scope.factors[i].risks[j].valueYes = ! $scope.factors[i].risks[j].valueNo;
      }else{
        $scope.factors[i].risks[j].valueNo  = ! $scope.factors[i].risks[j].valueYes;
      }

      return;
    }
  }

  $scope._saveFurther = function(testResults_id,emailRequested,email,emailSent){
    syncableDatabase.db().transaction( function(tx) {
      //get latest score id
      //insert these check box values against latest score id, plus email req. flags

      var params=[new Date().getTime(),testResults_id];
      var sql='INSERT INTO testResultsFurther ( date, testResults_id ,emailRequested, emailTo, emailDone, '

      if( emailRequested == true ){
        if( emailSent == true ){
          params=params.concat(1,email,1);
        }else{
          params=params.concat(1,email,0);
        }
      }else{
        params=params.concat(0,null,0);
      }

      for( var i=0;i < $scope.factors.length;i++ ){
        for( var j=0;j < $scope.factors[i].risks.length;j++ ){
          sql+=$scope.factors[i].risks[j].id+
            ( i==$scope.factors.length-1 && j==$scope.factors[i].risks.length-1 ?'':',');
          params=params.concat(
            typeof $scope.factors[i].risks[j].valueYes == 'boolean' &&
                   $scope.factors[i].risks[j].valueYes == true
          );
        }
      }

      var valueSql='';//one '?,' for every entry in params
      for(var i=0;i < params.length;i++){
        if( i == params.length-1 ){
          valueSql+='?';
        }else{
          valueSql+='?,';
        }
      }

      sql+=') values ('+valueSql+')';

      tx.executeSql(sql,params,
          null,
          function(tx,e){console.error('INSERT INTO testResultsFurther !ok'+e.message)});
    },
    function(e){ console.error('failed to save');console.dir(e); },
    function(){ syncableDatabase.sync(); }
    );
  }

   $scope.factors = QuestionsController.getQuestions('risk');

})

/* https://docs.angularjs.org/guide/forms */
.controller('CalcController', function($scope, $ionicPopover, $ionicPopup, $state, calcStateService,syncableDatabase,registerService,profileDialog,testService,wasLastResultTest) {

  $scope.isCalcTest = wasLastResultTest;

  $scope.$on('$ionicView.enter',function($event){
    testService.wasLastResultTest().then(function(res){$scope.isCalcTest=res;});
  });

  $scope.closePrefPopover = function() {
    $scope.testPrefModal.hide();
  }

  $scope.registered=registerService.registered();
  $scope.$on('event:registerChanged', function(e, rejection) {
    $scope.registered=registerService.registered();
  });

  $scope.showTestInfo = function(evt){
    testService.showDialog(evt,$scope.isCalcTest,{
      success:  function(newValue){
        $scope.isCalcTest = newValue;
      }
    });
  }

  $scope.$on('event:clearCalc', function(e, rejection) {
    $scope.reset();
  });

  $ionicPopover.fromTemplateUrl('templates/register.html', {
    scope: $scope
  }).then(function(popover) {
    $scope.mustRegPopover = popover;
  });

  $scope.$on('$destroy', function() {
    $scope.mustRegPopover.remove();
    $scope.within48hoursModal.remove();
  });

  $scope.master = {};
  $scope.$watch('master.hasAnswer',function(){
    if( $scope.master.hasAnswer == true ){
      syncableDatabase.db().transaction(function(tx) {
        tx.executeSql('INSERT INTO testResults ( date,type, baseline,current, score,within48hours, isTest) VALUES (?,?, ?,?, ?,?, ?)',[
            new Date().getTime(),
            'creatinine',

            $scope.master.baseline,
            $scope.master.current,

            $scope.master.answer,
            $scope.master.within48hours,

            $scope.isCalcTest==true?1:0
          ]);
      },
      function(e){ console.error('failed to save');console.dir(e); },
      function(){ syncableDatabase.sync(); }
      );
    }
  });

  $scope.goToNextStateFor=function(score){
    calcStateService.goToNextStateFor(score);
  }

  $scope.showRegister = function(evt){
    $scope.mustRegPopover.show(evt);
  }
  $scope.showProfile=function(evt){
    $scope.mustRegPopover.hide();
    profileDialog(evt);
  }

  $scope.update = function($event,calc) {
    $scope.master = angular.copy(calc); //will only have properties if valid

    /* if all properties, calc and output */
    if( $scope.master.baseline && $scope.master.current ){

      //$scope.master.answer=$scope.master.baseline + $scope.master.current;

      var diffPer=( $scope.master.current / $scope.master.baseline ) * 100;

      $scope.master.within48hours=false;

      if( diffPer >= 150 ){
        if( $scope.master.current > 354 ){
          $scope.master.answer=3;
        }else if ( diffPer>=300 ) {
          $scope.master.answer=3;
        }else if( diffPer>=200 ) {
          $scope.master.answer=2;
        }else{
          $scope.master.answer=1;
        }
        $scope.master.hasAnswer=true;
      }else{
        $scope.within48hoursModal.show($event);
      }

    }else{
    /* else display go button */
      $scope.master.answer=null;
      $scope.master.hasAnswer=false;
    }
  };

  $ionicPopover.fromTemplateUrl('templates/within48hours.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.within48hoursModal = modal;
  });

  $scope.within48hours = function() {
    $scope.master.within48hours=true;
    $scope.within48hoursModal.hide();
    if( ($scope.master.current - $scope.master.baseline) > 26 ){
      $scope.master.answer=1;
    }else{
      $scope.master.answer=0;
    }
    $scope.master.hasAnswer=true;
  };
  $scope.over48hours = function() {
    $scope.within48hoursModal.hide();
    $scope.master.hasAnswer=true;
    $scope.master.answer=0;
    $scope.master.showRepeatTestNote=true;
  };

  $scope.reset = function(form) {
    $scope.calc={};
    $scope.master = {};

    if ($scope.form) {
      // reset validation's state
      $scope.form.$setUntouched();
    }
  };

  $scope.resetAndBack= function(form){
    if( ! $scope.master.hasAnswer ){
      $scope.reset(form);
      $state.go("app.calc-type");
    }else{
      $scope.master.answer=null;
      $scope.master.hasAnswer=false;
    }
  };

   $scope.reset();
})

.controller('CalcTypeController',function($scope,$state, calcStateService,syncableDatabase, testPrefDialog,testService){
  $scope.showKnownMenu = false;

  $scope.$on('$ionicView.enter',function($event){
      $scope.showKnownMenu = false;
      testService.wasLastResultTest().then(function(res){$scope.isCalcTest=res;});
      testPrefDialog();
    });

  $scope.showTestInfo = function(evt){
    testService.showDialog(evt,$scope.isCalcTest,{
      success:  function(newValue){
        $scope.isCalcTest = newValue;
      }
    });
  }

  $scope.knownAki = function( score ){
    $scope.showKnownMenu = false;

    syncableDatabase.executeSql(
      'INSERT INTO testResults ( date, type, score, isTest) VALUES (?,?,?,?)',
      [
        new Date().getTime(),
        'known',
        score,
        $scope.isCalcTest==true?1:0
      ]
    ).then(function(results){
      calcStateService.goToNextStateFor(score);
    });
  }
})

.controller('CalcUrineController',function($scope,$state,$ionicPopover, calcStateService, registerService, profileDialog, syncableDatabase, urineOutputDialog, testService){
  $scope.$on('$ionicView.enter',function($event){
    $scope.calc = {};
    $scope.master = {};
    $scope.urineOutput={valueYes : null,valueNo : null};
    $scope.answer = -1;
    $scope.form.$setUntouched();
    testService.wasLastResultTest().then(function(res){$scope.isCalcTest=res;});
    urineOutputDialog();
  });

  $scope.$on('$destroy', function() {
    $scope.mustRegPopover.remove();
  });

  $scope.registered=registerService.registered();
  $scope.$on('event:registerChanged', function(e, rejection) {
    $scope.registered=registerService.registered();
  });

  $ionicPopover.fromTemplateUrl('templates/register.html', {
    scope: $scope
  }).then(function(popover) {
    $scope.mustRegPopover = popover;
  });

  $scope.itemClick = function(evt,q){
    var qSelect = $scope.urineOutput;
    if( evt.target.nodeName == 'INPUT' ){// only the check box itself

      //first click, honour what tapped on
      if( qSelect.valueYes == null &&
          qSelect.valueNo  == null
      ){
        if( evt.target.getAttribute('ng-model').indexOf('.valueNo') > -1 ){
          qSelect.valueYes = false;
        }else{
          qSelect.valueNo  = false;
        }
        //hackToFixHeightBug();

        return;
      }

      //which of yes and no was just clicked and changed value ? Toggle the other
      if( evt.target.getAttribute('ng-model').indexOf('.valueNo') > -1 ){
        qSelect.valueYes = ! qSelect.valueNo;
      }else{
        qSelect.valueNo  = ! qSelect.valueYes;
      }
      //hackToFixHeightBug();

      /* by Urine Specific code */
      if(qSelect.valueYes == true) {
        $scope.master.urine = -1;
        score(0);
      } else {
        $scope.answer = -1;
      }
      /* /by Urine Specific code */
      return;
    }
  }


  $scope.showTestInfo = function(evt){
    testService.showDialog(evt,$scope.isCalcTest,{
      success:  function(newValue){
        $scope.isCalcTest = newValue;
      }
    });
  }

  $scope.update = function($event,calc) {
    $scope.master = angular.copy(calc); //will only have properties if valid

    /* if all properties, calc and output */
    if( $scope.master.urine && $scope.master.hours ){
      if( $scope.master.urine == 0 && $scope.master.hours > 12 ){
        score(3);
        return;
      }
      if( $scope.master.urine < 0.3 && $scope.master.hours > 24 ){
        score(3);
        return;
      }
      if( $scope.master.urine < 0.5 && $scope.master.hours > 12 ){
        score(2);
        return;
      }
      if( $scope.master.urine < 0.5 && $scope.master.hours > 6 ){
        score(1);
        return;
      }

      score(0);
    }
  }

  $scope.goToNextStateFor=function(score){
    calcStateService.goToNextStateFor(score);
  }

  $scope.showRegister = function(evt){
    $scope.mustRegPopover.show(evt);
  }
  $scope.showProfile=function(evt){
    $scope.mustRegPopover.hide();
    profileDialog(evt);
  }

  function score( score ){
    syncableDatabase.executeSql(
      'INSERT INTO testResults ( date, type, urineRate,urineHours,score, isTest) VALUES (?,?, ?,?,?, ?)',
      [
        new Date().getTime(),
        'urine',

        $scope.master.urine,
        $scope.master.hours,
        score,

        $scope.isCalcTest==true?1:0
      ]
    ).then(function(){
      return syncableDatabase.executeSql("select max(id) as id from testResults");
    }).then(function(results){
      $scope.answer = score;
    });
  }
});
