"use strict";
angular.module('starter.controllerHistory', ['sync-service', 'starter.controllerQuestions'])

.filter('cfDate', function() {
	return function(input,def) {
		if( input == '' || input == undefined){
			return def;
		}
		return new Date(input);
	};
})

.controller('HistoryController', function($rootScope,$scope,$state,$ionicPopup,$location,$q,syncableDatabase,testService,wasLastResultTest,historyService) {

	$scope.dateFormat	= 'dd/MM/yy';
	$scope.timeFormat	= 'HH:mm';
	$scope.results 		= [];
	$scope.sortDir 		= 'ion-chevron-down';
	$scope.isCalcTest = wasLastResultTest;

	var sort = 'desc';

	$scope.$on('$ionicView.enter',function(){
		testService.wasLastResultTest().then(function(res){ $scope.isCalcTest = res });
		getHistory(sort);
	});

	$scope.showTestInfo = function(evt) {
		testService.showDialog(evt, $scope.isCalcTest, {
				success: function(newValue) {
				$scope.isCalcTest = newValue;
			}
		});
	}

	$scope.sortHistory = function() {
		if (sort == 'desc') {
			sort = 'asc';
			$scope.sortDir = 'ion-chevron-up';
		} else {
			sort = 'desc';
			$scope.sortDir = 'ion-chevron-down';
		}
		getHistory(sort);
	}

	$scope.scoreNA     = function (score) { return typeof score === 'undefined' };
	$scope.scoreRed    = function (score) { return score === 3; };
	$scope.scoreOrange = function (score) { return score === 2; };
	$scope.scoreYellow = function (score) { return score === 1; };
	$scope.scoreGrey   = function (score) { return score === 0; };

	var getHistory = function(sort) {
		historyService.getHistory().then(function(response){
			$scope.results = response
			$scope.results.sort(function(a,b){
				if (sort == 'desc') {
					return b.date-a.date
				} else {
					return a.date-b.date
				}
			});
		})
	}
})

.controller('HistoryDetailController', function($rootScope,$scope,$state,$ionicPopup,$q,syncableDatabase,QuestionsController,emailService,registerService,$ionicLoading,createEmailDialog,historyService) {
	$scope.dateFormat		= 'dd/MM/yy';
	$scope.timeFormat		= 'HH:mm';
	$scope.detail				= [];

	$scope.hideRefYes		= false;
	$scope.hideRefNo		= false;
	$scope.hideRiskYes	= false;
	$scope.hideRiskNo		= false;

	var id 		= $state.params.detail
	var table = $state.params.table
	historyService.getDetail(table,id).then(function(response) {
		$scope.detail = response[0];
		if (table == 'testResults') {
			$scope.detail.furtherTests = response[1];
			$scope.detail.refer        = response[2];
			$scope.detail.steps        = response[3];
			$scope.detail.stepsTreat   = response[4];
			$scope.detail.risk         = response[5];
			countRef();
			countRisk();
			setTreatmentLocation();
		} else {
			$scope.detail.refer        = response[1];
			$scope.detail.steps        = response[2];
			$scope.treatmentLocation 	 = 'unknown';
			countRef();
		}
	})

	$scope.email_loading = false
	$scope.sendEmail = function(evt) {
		createEmailDialog(evt,{
			success: function(email) {
				console.log('Simple modal closed '+email);
				$scope.email_loading = true
				$ionicLoading.show({
					template: 'Sending email...',
					cssClass: 'review'
				});
				var template = 'referralHistory'
				var data = $scope.detail
				filterData(data)
				emailService.sendEmail(template,email,data).then(function(response){
					$ionicLoading.hide();
					$scope.email_loading = false
					if (response.ok) {
						$ionicPopup.alert({
							title: 'Email sent',
							template: 'The email will be with you shortly',
							cssClass: 'further ok',
							okType: 'button-balanced'
						});
					} else {
						$ionicPopup.alert({
							title: 'Email sending delay',
							template: 'The email will be with you after a short delay',
							cssClass: 'further delay',
							okType: 'button-balanced'
						});
					}
				})
			}
		})
	};

	var filterData = function(data) {
		if (typeof data.title != 'undefined') {
			if (data.title == 'On supplementary') {
				delete data.units
				data.title = 'On supplementary oxygen?'
			}
			if (data.title == 'Inspired oxygen' && data.value == null) {
				delete data.units
			}
		}
		for (var key in data) {
			if (Array.isArray(data[key])) {
				for (var i = 0; i < data[key].length; i++) {
					filterData(data[key][i])
				}
			} else if (key == 'units' && data[key] == null) {
				delete data[key]
			} else if (key == '$$hashKey') {
				delete data[key]
			} else if (data[key] == null || typeof data[key] == 'null') {
				data[key] = 'N/A'
			} else if (data[key] == 'true') {
				data[key] = 'Yes'
			} else if (data[key] == 'false') {
				data[key] = 'No'
			}
		}
		if (data.type == 'N/A') {
			delete data.type
		}
	}

	var countRef = function() {
		var yesCount = 0;
		var noCount  = 0;
		for (var i = 0; i < $scope.detail.refer.length; i++) {
			if ($scope.detail.refer[i].value == 'true') {
				yesCount++;
			} else if ($scope.detail.refer[i].value == 'false') {
				noCount++;
			}
		};
		if (yesCount == 0) {
			$scope.hideRefYes = true;
		} else if (noCount == 0) {
			$scope.hideRefNo = true;
		};
	}

	var countRisk = function() {
		var yesCount = 0;
		var noCount  = 0;
		for (var i = 0; i < $scope.detail.furtherTests.length; i++) {
			if ($scope.detail.furtherTests[i].value == 'true') {
				yesCount ++;
			} else if ($scope.detail.furtherTests[i].value == 'false') {
				noCount ++;
			}
		};
		if (yesCount == 0) {
			$scope.hideRiskYes=true;
		} else if (noCount == 0) {
			$scope.hideRiskNo=true;
		};
	}

	var setTreatmentLocation = function() {
		var gmCount = 0;
		var cmCount = 0;
		var tlRes = '';
		for (var i = 0; i < $scope.detail.stepsTreat.length; i++) {
			if ($scope.detail.stepsTreat[i].title == 'North West Coast Strategic Clinical Network') {
				for (var j = 0; j < $scope.detail.stepsTreat[i].tests.length; j++) {
					if ($scope.detail.stepsTreat[i].tests[j].value =='true') {
						cmCount++;
					};
				};
			} else {
				for (var k = 0; k < $scope.detail.stepsTreat[i].tests.length; k++) {
					if ($scope.detail.stepsTreat[i].tests[k].value =='true') {
						gmCount++;
					};
				};
			};
		};
		if (cmCount > gmCount) {
			tlRes = 'cm';
		} else if (cmCount < gmCount) {
			tlRes = 'gm';
		} else {
			tlRes = 'unknown';
		};
	}

});
