"use strict";
angular.module('starter.controllerpdf', ['sync-service','email-ask.service',"email-service",'register-service','profile-dialog','pdf-service'])

.controller('PdfController', function($rootScope,$scope,$http,$window,$state,$stateParams,$ionicLoading,PdfService, createEmailDialog, emailService) {

  $scope.prefix = '';
  $scope.currentContent='loading';

  var path = '';
  var pdfIndex = Number($stateParams.pdfIndex);
  var pageIndex = Number($stateParams.pageIndex);

  var pdf = PdfService.getBook(pdfIndex);
  //console.log(pdf);
  var prefix = 'doc/'+(Number(pdfIndex)+1)+' '+pdf.name+'/' ;
  var currentPage = pdf.pages[pageIndex];
  //var chapterPos = currentPage.split(' ')[0];

  var pdfInfo = {name: 'Patient Information Leaflet on AKI', url: pdf.url};

  $scope.hasNext     = pageIndex<pdf.pages.length-1;
  $scope.hasPrevious = pageIndex>0;
  //$scope.hasParent   = chapterPos.indexOf('.') != -1;
  $scope.pdfname = pdf.name

  $scope.currentPageNumber = pageIndex + 1;
  $scope.pageCount   = pdf.pages.length;

  loadPage( currentPage );

  $scope.pdfNameToCssClass = function pdfNameToCssClass(pdfname) {
    return pdfname.replace(/ /g,'-');
  }

  $scope.goToURL = function goToURL(url){
    var ref = window.open(url, '_system', 'location=yes');
  }

  $scope.next = function next(){
//console.log('next '+(pageIndex+1));
    $state.go('app.pdf',{pdfIndex:pdfIndex,pageIndex:pageIndex+1});
    $rootScope.previousState = undefined; //clear the cache of where we were so previous() works on next state
  }

  $scope.previous = function previous(){
    /*
    if we came from a link in a page, we want to go back to there, not just page -1 but only if link in same book
    https://basecamp.com/2230587/projects/11714560/todos/263843301
  */
    if( typeof $rootScope.previousState != 'undefined' && $rootScope.previousStateParams.pdfIndex == $stateParams.pdfIndex ){
      $state.go( $rootScope.previousState, $rootScope.previousStateParams);
    }else{
      $state.go('app.pdf',{pdfIndex:pdfIndex,pageIndex:pageIndex-1});
    }
    $rootScope.previousState = undefined; //clear the cache of where we were so previous() works on next state
  }

  $scope.sendEmail = function( evt ){
    var plTrue = "patientLeafletSend";
    createEmailDialog( evt,{
      success:  function(email){
        $ionicLoading.show({
          template: 'Sending email...',
        });

        emailService.sendEmail('patientLeaflet',email,pdfInfo)
        .then(function(data,status){
          $ionicLoading.hide();
        })
        .catch(function(data,status){
          $ionicLoading.hide();
          console.error("failed to call "+data);
          $ionicPopup.alert({
            title: 'Could not find server' ,
            template: 'Are you online ?'+data,
            cssClass: 'refer error review',
            okType: 'button-balanced'
          });
        });
      }
    },
    plTrue);
  }

  function loadPage(nam){
//console.log( 'loadPage '+nam  )    ;
    var pth = PdfService.getPathTo(pdfIndex,pageIndex);

    //name of this chapter
    var n = PdfService.fileNameToTitle( nam );

    $scope.prefix = n ;// book.name+ ' > '+ pth.join(' : ') +' > '+ n;

//console.log( 'loadPage set prefix '+prefix);
    $scope.currentContent='loading ';

    $http.get(prefix+nam)
    .then(function(res){
//console.log( 'loadPage req then');

      var text = res.data;

      $scope.currentContent=text;
    })
    .catch(function(e){
      console.log('loadPage $http 1 failed');
      console.dir(e);
    })
//console.log( 'loadPage req done '+prefix+nam);
  }

})
.directive('compile', ['$compile',function($compile) {
  //http://stackoverflow.com/questions/17417607/angular-ng-bind-html-unsafe-and-directive-within-it
  return function(scope, element, attrs) {
      scope.$watch(
          function(scope) {
              return scope.$eval(attrs.compile);
          },
          function(value) {
              element.html(value);
              var doCompile = $compile(element.contents())(scope);
              scope.$emit('compiled');
          }
      );
  };
}])
.directive('slider', [function () {
  return function(scope, element, attrs) {

    var el           = angular.element(element);
    var slides       = el.children();
    var slidesCount  = slides.length;
    var activeSlide  = slides[0];
    var prevSlide    = false;
    var nextSlide    = false;
    var slideHeight  = 0;

    var btns = angular.element('<div class="slider-btns slider-btns-left"><i class="icon ion-ios-arrow-left"></i></div><div class="slider-count">'+'1 of '+slidesCount+'</div><div class="slider-btns slider-btns-right"><i class="icon ion-ios-arrow-right"></i></div>');

    btns.on('click',function(e){ // click next or prev btn
      var thisIs = angular.element(this);
      if( thisIs.hasClass('slider-btns-left') ) {
        slidePrev();
      } else {
        slideNext();
      }
    });

    for (var i = 0; i < slides.length; i++) {
      angular.element(slides[i]).data('slide-index',i);

      angular.element(slides[i]).on('swipeleft',function(e){
        var thisIs = angular.element(this);
        if( thisIs.data('slide-index') < slides.length-1 ){
          slideNext();
        }
      });
      angular.element(slides[i]).on('swiperight',function(e){
        var thisIs = angular.element(this);
        if( thisIs.data('slide-index') > 0 ){
          slidePrev();
        }
      });
    }

    function getActiveSlide () {
      for (var i = 0; i < slides.length; i++) {
        if ( angular.element(slides[i]).hasClass('slide-active') ) {
          if (i > 0) {
            prevSlide = slides[i-1];
            angular.element(btns[0]).removeClass('slider-btns-disabled');
          } else {
            prevSlide = false;
            angular.element(btns[0]).addClass('slider-btns-disabled');
          }
          activeSlide = slides[i];
          slideHeight  = activeSlide.offsetHeight;
          if (i < slides.length-1) {
            nextSlide = slides[i+1];
            angular.element(btns[2]).removeClass('slider-btns-disabled');
          } else {
            nextSlide = false;
            angular.element(btns[2]).addClass('slider-btns-disabled');
          }
        angular.element(btns[1]).text(i+1+' of '+slidesCount);
        };
      };
    }

    function slidePrev () {
      if ( angular.element(btns[0]).hasClass('slider-btns-disabled') ) {
        return false;
      } else {
        angular.element(activeSlide).removeClass('slide-active').addClass('slide-right');
        angular.element(prevSlide).removeClass('slide-left').addClass('slide-active');
        getActiveSlide();
        initSlider();
      };
    }

    function slideNext () {
      if ( angular.element(btns[2]).hasClass('slider-btns-disabled') ) {
        return false;
      } else {
        angular.element(activeSlide).removeClass('slide-active').addClass('slide-left');
        angular.element(nextSlide).removeClass('slide-right').addClass('slide-active');
        getActiveSlide();
        initSlider();
      };
    }

    function initSlider () {
      el.css('height',slideHeight+'px'); // set slider height
    }

    element.append( btns );

    getActiveSlide();
    initSlider();
  }
}])

;
