"use strict";
angular.module("starter.controllerQuestions", [])
.factory('QuestionsController',function(){
return {

	getQuestions:function( question ){
		switch(question){
			case 'risk':
				var risks = [
					{title:'Baseline risk factor',risks:[
						{id:'age',label:'Age ≥ 65 years',valueYes:null,valueNo:null},
						{id:'baseline',label:'Chronic Kidney Disease',valueYes:null,valueNo:null},
						{id:'previousAKI',label:'History of AKI',valueYes:null,valueNo:null},
						{id:'diabetes',label:'Diabetes',valueYes:null,valueNo:null},
						{id:'heart',label:'Heart failure',valueYes:null,valueNo:null},
						{id:'liver',label:'Liver disease',valueYes:null,valueNo:null},
						{id:'dementia',label:'Neurological/Cognitive impairment limiting fluids',valueYes:null,valueNo:null},
						{id:'nephrotoxic',label:'Potential nephrotoxic medicines',valueYes:null,valueNo:null},
						{id:'contrast',label:'Iodinated contrast in past week',valueYes:null,valueNo:null}
					]},
					{title:'Current illness risk factor',risks:[
						{id:'earlywarning',label:'Deteriorating Early Warning Score',valueYes:null,valueNo:null},
						{id:'hypovolaemia',label:'Hypovolaemia',valueYes:null,valueNo:null},
						{id:'sepsis',label:'Sepsis',valueYes:null,valueNo:null},
						{id:'newonset',label:'New onset or worsening of urological symptoms',valueYes:null,valueNo:null}
					]}
				];
				return risks
			break;

			case 'criteria':
				var criteria = [
					{ id:'aki',value:false,label:'AKI 3'},
					{ id:'ckd',value:false,label:'Known CKD stage 4/5'},
					{ id:'transplant',value:false,label:'Renal transplant'},
					{ id:'haematuria',value:false,label:'Dipstick haematuria'},
					{ id:'proteinurea',value:false,label:'Dipstick proteinuria'},
					{ id:'nonresolving',value:false,label:'Non-resolving AKI'},
					{ id:'noclear',value:false,label:'AKI with no clear cause'}
				];
				return criteria
			break;

			case 'steps':
				var steps = [
					{title:'Metabolic',tests:[
						{id:'pot',value:null,label:'Potassium','units':'mmol/L',type:'number',min:"1",max:"9",transferMin:3,transferMax:6},
						{id:'ph',value:null,label:'pH',type:'number',min:6.7,max:7.7,transferMin:7.20},
						{id:'bicarbonate',value:null,label:'Bicarbonate','units':'mmol/L',type:'number',min:10,max:40,transferMin:16}
					]},
					{title:'Physiological',tests:[
						{id:'rate',value:null,label:'Respiratory rate','units':'per min.',type:'number',min:5,max:60,transferMin:9,transferMax:25},
						{id:'copd',value:null,valueYes:null,valueNo:null,label:'Known COPD?',type:'yn',modify:{
							id:'sat',
							false:{
								transferMin : 93,
								warnMin: undefined,
								warnMax: undefined
							},
							true:{
								transferMin : 87,
								warnMin: 87,
								warnMax: 94
							}
						}},
						{id:'sat',value:null,label:'Oxygen saturation','units':'%',type:'number',min:50,max:100,transferMin:93.0},
						{id:'supp',value:null,label:'On supplementary',units:'oxygen',type:'bool',controls:'insp'},
						{id:'insp',value:null,label:'Inspired oxygen','units':'%',type:'number',min:24,max:100,transferMax:35.01,controlledBy:'supp',hide:true},
						{id:'cpap',value:null,valueYes:null,valueNo:null,label:'CPAP / BiPAP in last 24 hours',type:'yn'},
						{id:'pulse',value:null,label:'Pulse','units':'bpm',type:'number',min:30,max:180,transferMin:49,transferMax:121},
						{id:'press',value:null,label:'Systolic pressure','units':'mmHg',type:'number',min:60,max:250,transferMin:99.01},
						{id:'vaso',value:null,valueYes:null,valueNo:null,label:'Vasopressors last 24 hours',type:'yn'},
						{id:'level',value:null,label:'Conscious level',type:'avpu',transfer:'Alert'}
					]}
				];
				return steps
			break;

			case 'stepsTreat':
				var steps = [
				    {title:'Fluid balance',tests:[
				      {id:'fluid',value:'',label:'Clinically assess fluid balance',help:'Review pulse and blood pressure charts. Examine legs and sacrum for dependent oedema, chest for evidence of pulmonary oedema, neck for evidence of raised JVP and auscultate heart looking for gallop rhythm. '},
				      {id:'chart',value:'',label:'Initiate fluid balance charts',help:'Chart fluid intake and output from all sources. If possible weigh the patient daily. Discuss this with the patient and the nursing team.'},
				      {id:'urine',value:'',label:'Document urine dipstick',help:'Perform urine dipstick and document results clearly in medical notes. If there is evidence of haematuria/proteinuria consider immunology screen and referral to renal services. If 2+ or greater of protein send spot urine protein : creatinine ratio.'}
				    ]},
				    {title:'Exclude post-renal cause',tests:[
				      {id:'ultrasound',value:'',label:'Order renal ultrasound',help:'If there is no identified cause of AKI order ultrasound to be performed within 24 hours.'},
				      {id:'catheter',value:'',label:'Consider urinary catheter',help:'Consider urinary catheter if there is a high suspicion of lower urinary tract obstruction. Catheterisation should not be required to monitor fluid balance.'}
				    ]},
				    {title:'Review medications',tests:[
				      {id:'stop',value:'',label:'Stop nephrotoxic drugs',help:'Suspend ACE inhibitors and angiotensin blockers, diuretics and non-­‐steroidal anti-­‐inflammatories.'},
				      {id:'doses',value:'',label:'Review drug doses',help:'Review doses of all other medications, especially antibiotics. Note that these may need further adjustment if function worsens/improves.'},
				      {id:'lmwh',value:'',label:'Review LMWH',help:'If eGFR <30ml/min/1.73m<sup>2</sup> will need dose reduction.'}
				    ]},
				    {title:'Further care',tests:[
				      {id:'bloods',value:'',label:'Order repeat bloods',help:'If haematuria or proteinuria consider checking ANCA, anti-­GBM, C3, C4, rheumatoid factor, ANA,ENA (including ds‐DNA), immunoglobulin levels and serum and urine electrophoresis.<br>Order repeat renal chemistry for the next day including bicarbonate and chloride.'}
				    ]},
				    {title:'North West Coast Strategic Clinical Network',tests:[
				      {id:'cause',value:'',label:'Treat underlying cause',help:'<u>Consider the following:</u><br>Hypoperfusion<br>Obstruction<br>Sepsis<br>Intrinsic Kidney Diseases:<ul><li>Glomerulonephritis</li><li>Vasculitis</li><li>Myeloma</li><li>HUS</li></ul>Rhabdomyolysis<br>Toxins (drugs or contrast)<br>Trauma<br>Surgery<br>Liver decompensation'},
				      {id:'reqBloods',value:'',label:'Request bloods',help:'<u>Consider the following:</u><br>U&E<br>Bicarbonate<br>FBC<br>Coagulation Profile<br>Blood gas<br>Creatinine Kinase<br>Bone profile<br>Chloride<br>LFT<br>Blood Culture & Lactate if sepsis is suspected'},
				      {id:'dipstick',value:'',label:'Document urine dipstick',help:'Perform urine dipstick for blood, protein, leukocytes, nitrites and glucose.<br><br>Haematuria or proteinuria on dipstick in the absence of urinary tract infection or trauma may suggest acute nephritis.<br><br>Haematuria with symptoms of obstruction may indicate a renal tumour or stone.'},
				      {id:'fluidStatus',value:'',label:'Document fluid status',help:'<u>Review the following:</u><br>Capillary refill<br>Pulse<br>BP (check postural)<br>JVP<br>Respiratory rate<br>Oedema (pulmonary/peripheral)<br>Passive leg raise<br>Weight'},
				      {id:'management',value:'',label:'Fluid management plan',help:'Once the fluid balance status is assessed, IV fluids should be considered. Choose one of the following protocols:<ul><li>Resuscitation</li><li>Replacement</li><li>Routine Maintenance</li></ul>Daily fluid requirement is 25-30ml/kg/day of water. 1mmol/kg of Na+, K+, Cl- and 50-100g of Glucose'},
				      {id:'balance',value:'',label:'Initiate fluid balance charts',help:'Chart all fluid intake and output from all sources.<br><br>Weigh patient daily (at least twice weekly).<br><br>Encourage patient where possible to be involved in fluid charting.'},
				      {id:'reqUltrasound',value:'',label:'Request renal ultrasound',help:'If obstruction suspected perform within 24 hours.<br><br>If no cause identified or if at risk of obstruction perform with 24hours.<br><br>If pyonephrosis suspected perform within 6 hours.<br><br>Do not routinely offer ultrasound when cause of AKI has been identified.'},
				      {id:'urineCatheter',value:'',label:'Consider urinary catheter',help:'Consider catheter if there is a high suspicion of urinary tract obstruction.<br><br>Catheterisation to measure urine output should not be required.<br><br>Urinary catheters inserted for measurement of output should be removed promptly when no longer necessary.'},
				      {id:'nephrotoxic',value:'',label:'Stop nephrotoxic drugs',help:'Suspend all nephrotoxic potential drugs and drugs that can become toxic in AKI <ul><li>ACE-Inhibitors</li><li>Angiotensin Receptor Blockers</li><li>Aliskiren</li><li>NSAIDs</li><li>COX 2 inhibitors</li><li>Diuretics</li><li>Metformin</li><li>Trimethoprim</li></ul>'},
				      {id:'reviewDoses',value:'',label:'Review drug doses',help:'Review doses of all other medications.<br><br>LMWH requires dose reduction if eGFR < 30.<br><br>Many antibiotics may need a dose adjustment during AKI and a further adjustment if function worsens/improves.<br><br>All drugs must be further reviewed with an explicit plan on discharge'},
				      {id:'disease',value:'',label:'Intrinsic renal disease bloods',help:'If dipstick reveals haematuria or proteinuria in the absence of UTI or trauma request:<ul><li>ANCA</li><li>ANA</li><li>Complement C3 & C4</li><li>Anti GBM</li><li>Serum Electrophoresis</li><li>Urine Bence Jones protein</li></ul>If platelets low request blood film and LDH (stool culture if bloody stools)<ul><li>If dialysis needed request:</li><li>Hepatitis B Surface Antigen</li><li>Hepatitis C Antibody</li><li>HIV serology</li></ul>'}
				    ]}
				   ];
				return steps
			break;

			default:
				return 'fail'
		}
	}
}

});
