"use strict";
angular.module('starter.controllerrefer', ['sync-service','evCareCenterDirective','geo-service','email-ask.service',"email-service",'ngToggle', 'starter.controllerQuestions','test-preferences'])

.controller('ReferResultController', function($rootScope,$scope,$state,$ionicLoading,$ionicPopup,$ionicPopover,$q, geoService,createEmailDialog,emailService,syncableDatabase) {
	$scope.safe = $state.params.safe;

	$scope.hasPosition=false;

	$scope.regions = geoService.getRegions();

	//will normally have cached from home screen.
	var r = geoService.getCurrentRegion();
	if( r ){
		setUpRegion( r );
		if( geoService.hasPosition() ){
			var p = geoService.getPosition();
			$scope.latitude    = p.latitude;
			$scope.longitude   = p.longitude;
			$scope.hasPosition = true;
		}else{
			geoService.get()
		    .then(function(p){
				$scope.latitude    = p.latitude;
				$scope.longitude   = p.longitude;
				$scope.hasPosition = true;
		    });
		}

	}else{
		//but when refreshing page in development
		geoService.get()
	    .then(function(position){
			setUpRegionForPosition( position );
	    });
	}

	$ionicPopover.fromTemplateUrl('templates/popover-region.html',{
		scope : $scope
	})
	.then(function(popover){
		$scope.regionPopover = popover;
	});

	$scope.showRegionChooser = function( e ){
		$scope.regionPopover.show( e );
	}

	$scope.setCurrentRegion = function( r ){
		$scope.regionPopover.hide();
		$scope.currentRegion = r;
		geoService.setCurrentRegion( r );
		setUpRegion( r );
	}

	$scope.sendEmail=function(evt){
		createEmailDialog(evt,{
          success:  function(email){
            $ionicLoading.show({
              template: 'Sending email...',
              cssClass: 'review'
            });

          var referSafetyResults;

		  syncableDatabase.executeSql("select * from referSafety where id=? order by date desc",[ $state.params.referSafety_id ])
		  .then(function( results ){
		  	referSafetyResults = results;
		  	return syncableDatabase.executeSql("select * from refer where id=? order by date desc",[ referSafetyResults.rows.item(0).refer_id ])
		  })
		  .then(function( referResults ){
			var data={
				criteria:syncableDatabase.resultRowToStruct( referResults.rows.item(0) ),
				transfer:syncableDatabase.resultRowToStruct( referSafetyResults.rows.item(0) )
			}

            if( referResults.rows.item(0).testResults_id != -1 ){ //if( has-full ){
            	$q.all([
            		syncableDatabase.executeSql("select * from testResults where id=? order by date desc",[ referResults.rows.item(0).testResults_id ]),
            		syncableDatabase.executeSql("select * from riskAssesment where testResults_id=? order by date desc",[ referResults.rows.item(0).testResults_id ]),
            		syncableDatabase.executeSql("select * from treatment where testResults_id=? order by date desc",[ referResults.rows.item(0).testResults_id ])
            	])
            	.then(function( results ){
            		var testResultsResults   = results[0];
            		var riskAssesmentResults = results[1];
            		var treatmentResults     = results[2];

	            	data.calculation={
						score         : testResultsResults.rows.item(0)
					};
					data.emergencies=syncableDatabase.resultRowToStruct( riskAssesmentResults.rows.item(0) );
					data.assessment=syncableDatabase.resultRowToStruct( treatmentResults.rows.item(0) );

					data.region = $scope.currentRegion.name;

					sendEmail(email,data);
			    });//end refer queries via calc
			}else{
				sendEmail(email,data);
			}

          }); // end $q.all

	  }});//end createEmailDialog success handler
	}//end $scope.sendEmail

	$scope._saveEmail=function(email,emailSent,referSafety_id){
    	syncableDatabase.db().transaction( function(tx) {
    		var params=[new Date().getTime(),$state.params.referSafety_id,email,emailSent];
    		tx.executeSql(
    			'INSERT INTO referEmail ( date, referSafety_id ,emailTo, emailDone, region) values (?,?,?,?,?)',
    			[
    				new Date().getTime(),
    				referSafety_id,
    				email,
    				emailSent,
    				$scope.currentRegion.name
    			],
    			null,
    			function(tx,e){console.error('INSERT INTO referEmail !ok'+e.message)}
    		);
    	},
	    function(e){ console.error('failed to save');console.dir(e); },
	    function(){ syncableDatabase.sync(); }
	    );


      	$state.go('app.home');
    };

    function sendEmail(email,data){
		emailService.sendEmail('referalResult_3',email,data)
		.then(function(data,status){
			$ionicLoading.hide();
			if(data.ok == true ){
			// - set email done flag
			  $scope._saveEmail(email,true,$state.params.referSafety_id);
			  $ionicPopup.alert({
			    title: 'Email sent',
			    template: 'The email will be with you shortly',
			    cssClass: 'refer ok review',
				okType: 'button-balanced'
			  });
			}else{
			  console.error("remote error");
			  //as error()
			  $ionicPopup.alert({
			    title: 'Email sending delay',
			    template: 'The email will be with you after a short delay',
			    cssClass: 'refer warn review',
				okType: 'button-balanced'
			  });
			  $scope._saveEmail(email,false,$state.params.referSafety_id);
			}
		})
		.catch(function(data,status){
			$ionicLoading.hide();
			console.error("failed to call "+data);
			$ionicPopup.alert({
				title: 'Could not find server' ,
				template: 'The email will be sent next time you go online',
				cssClass: 'refer error review',
				okType: 'button-balanced'
			});
			$scope._saveEmail(email,false,$state.params.referSafety_id);
		});
	}

	function setUpRegionForPosition( p ){
		$scope.latitude    = p.latitude;
		$scope.longitude   = p.longitude;
		$scope.hasPosition = true;

		$scope.currentRegion   = geoService.getRegionForPosition( p );
		setUpRegion( $scope.currentRegion );
	}

	function setUpRegion ( r ){
		$scope.locations    = r.hospitals;
		$scope.postcodeData = r.postcodeData;
	}
})

.controller('ReferSafetyController', function($scope,$state, syncableDatabase, QuestionsController) {
	$scope.shouldEnableContinue=function(){
		if( ! $scope.steps ){
			return false;
		}
		return _allNotNull( $scope.steps );
	}

	$scope.continue=function(){
		var isSafe=_allSafe( $scope.steps );

		syncableDatabase.db().transaction(function(tx) {
			var params=[new Date().getTime(),$state.params.testResults_id,$state.params.refer_id,isSafe ];

	    	var sql='insert into referSafety (date,testResults_id,refer_id,isSafe';

			for(var j=0;j<$scope.steps.length;j++){
				var tests=$scope.steps[j].tests;
				for(var i=0;i<tests.length;i++){
					var t=tests[i];
					sql+=','+t.id;
					switch( t.type ){
						case 'yn':
							params=params.concat( t.valueYes );
							break;
						case 'bool':
							params=params.concat( t.valueBool );
							break;
						case 'avpu':
							params=params.concat( t.valueString );
							break;
						case 'number':
							params=params.concat( t.value );
							break;
					}
		        }
		    }

			sql+=') values (';

			var valueSql='';//one '?,' for every entry in params
			for(var i=0;i < params.length;i++){
				if( i == params.length-1 ){
				  sql+='?';
				}else{
				  sql+='?,';
				}
			}

			sql+=')';

			tx.executeSql(
				sql,params,
				function(){
					syncableDatabase.sync();

					syncableDatabase.db().transaction(function(tx) {
						tx.executeSql('select max(id) as id from referSafety',[], function (tx,results){
							$state.go('app.refer-result',{safe:isSafe,referSafety_id:results.rows.item(0).id});
						});
					});

				    //reset
				    $scope.steps = $scope.stepsMaster;

				},
				function(tx,e){console.error('failed to save');console.dir(e);}
			);
		});
	}

	$scope.steps = QuestionsController.getQuestions('steps');

	$scope.stepsMaster=$scope.steps ;

	$scope.itemClick=function(evt){
		if(  evt.target.nodeName == 'INPUT' ){
			for( var idx=0;idx < $scope.steps.length;idx++ ){
				if( $scope.steps[idx].title == evt.currentTarget.getAttribute('data-step-title')  ){
					var test  = $scope.steps[idx].tests[ evt.currentTarget.getAttribute('data-test-idx') ];
					if( typeof test.controls != 'undefined' ){
						var tests=$scope.steps[ idx ].tests;
						for(var j=0;j<tests.length;j++){
							if( tests[j].id == test.controls ){
								tests[j].hide = !test.valueBool;
								tests[j].controllerValueBool=test.valueBool;
								if( typeof tests[j].value == 'undefined' ){
									return;
								}
							}
						}
					}
				}
			}
		}

		var tests=[];
		var stepidx=0;
		for( var stepsIdx=0;stepsIdx < $scope.steps.length;stepsIdx++ ){
			if( $scope.steps[stepsIdx].title == evt.currentTarget.getAttribute('data-step-title')  ){
				tests=$scope.steps[stepsIdx].tests;
				stepidx=stepsIdx;
			}
		}

		for( var i=0;i < tests.length;i++ ){
		  if( tests[i].id == evt.currentTarget.getAttribute('data-model-id') ){
		    if( tests[i].type == 'yn' ){

    			if(  evt.target.nodeName == 'INPUT' ){// only the check box itself
					//console.log(tests[i].id +'@'+i+' y:'+tests[i].valueYes+' n:'+tests[i].valueNo);

					//first click, honour what tapped on
					if( tests[i].valueYes == null &&
						tests[i].valueNo  == null
					){
						if( evt.target.getAttribute('ng-model').indexOf('valueNo') > -1 ){
							tests[i].valueYes = false;
						}else{
							tests[i].valueNo  = false;
						}
						return;
					}

					//which of yes and no was just clicked and changed value ? Toggle the other
					if( evt.target.getAttribute('ng-model').indexOf('valueNo') > -1 ){
						tests[i].valueYes = ! tests[i].valueNo;
					}else{
						tests[i].valueNo  = ! tests[i].valueYes;
					}
					//console.log(tests[i].id +'@'+i+' y:'+tests[i].valueYes+' n:'+tests[i].valueNo);
				}

				if( typeof tests[i].modify != 'undefined' ){
					for( var stepsIdx=0;stepsIdx < $scope.steps.length;stepsIdx++ ){
						for(var j=0;j<$scope.steps[stepsIdx].tests.length;j++){
							if( $scope.steps[stepsIdx].tests[j].id == tests[i].modify.id ){
								if( typeof tests[i].modify[ tests[i].valueYes ].transferMin != 'undefined' ){
									$scope.steps[stepsIdx].tests[j].transferMin = tests[i].modify[ tests[i].valueYes ].transferMin;
								}
								if( typeof tests[i].modify[ tests[i].valueYes ].warnMin != 'undefined' ){
									$scope.steps[stepsIdx].tests[j].warnMin = tests[i].modify[ tests[i].valueYes ].warnMin;
									//console.log('set warnMin');
								}else{
									delete $scope.steps[stepsIdx].tests[j].warnMin;
									//console.log('unset warnMin');
								}
								if( typeof tests[i].modify[ tests[i].valueYes ].warnMax != 'undefined' ){
									$scope.steps[stepsIdx].tests[j].warnMax = tests[i].modify[ tests[i].valueYes ].warnMax;
									//console.log('set warnMax');
								}else{
									delete $scope.steps[stepsIdx].tests[j].warnMax;
									//console.log('unset warnMax');
								}
							}
						}
					}
				}
			}
		    return;
		  }
		}
	}

	$scope.isVisible=function(test){
		if(test.type == 'avpu'){
			return 'ng-hide';
		}
		if( typeof test.controlledBy != 'undefined' ){
			return test.hide?'ng-hide':'ng-show';
		}

		return 'ng-show';
	}

	$scope.isWarn=function(test){
		if( typeof test.warnMin == 'undefined' ){
			return 'no-warn';
		}
		return test.value > test.warnMin && test.value < test.warnMax ? 'warn' : 'no-warn';
	}

	$scope.isSafe=function(test,modelField){
//console.debug(test.id);
		if( typeof modelField == 'undefined' ){
			return 'safe';
		}
		if( typeof test.modify != 'undefined' ){
			return 'safe';
		}

		if( test.type == 'yn' ){
			return test.valueYes==true?'unsafe':'safe';
		}

		if( test.type == 'avpu' ){
			return (typeof test.valueString=='undefined'||test.valueString=='Alert')?'safe':'unsafe';
		}

		if( test.type == 'bool' && typeof test.controls == 'undefined'){
			return !test.valueBool?'safe':'unsafe';
		}
		if( !modelField.$pristine && modelField.$valid ){
//console.debug('? '+modelField.$pristine+' '+modelField.$valid);
			return _testSafe(test)?'safe':'unsafe';
		}
		return 'safe';
	}

	function _allNotNull( steps ){
		if( typeof steps == 'undefined' ) return false;

		var cnt=0;
		var length=0;
		for(var j=0;j<steps.length;j++){
			var test=steps[j].tests;
			length+=test.length;
			for(var i=0;i<test.length;i++){
//console.debug('@'+j+' '+i+' '+test[i].id);
				if( typeof test[i].controlledBy == 'undefined' ){
					if(
						 test[i].value       != null ||
						 test[i].valueString != null ||
						 test[i].type        =='bool'||
						(test[i].type        =='yn' && ( test[i].valueYes!=null || test[i].valueNo!=null ))
					){
						cnt++;
//console.debug(' filled');
					}
				}else{
					for(var j=0;j<test.length;j++){
						if( test[j].id == test[i].controlledBy ){
							//if hidden, is valid
							//if shown, check value
							if( test[j].valueBool != 1 ){
								cnt ++;
//console.debug(' filled (!=1)');
							}else{
								if( test[i].value != null ){
									cnt++;
//console.debug(' filled');
								}
							}
						}
					}
				}
			}
		}
//console.info('all not null '+(cnt==length)+ ' completed: '+cnt+'/'+length);
		return cnt == length;
	}

	function _testSafe( test ){
		if( typeof test.controls != 'undefined' ){
			return true;
//console.info( test.id+' safe (control)');
		}else if( typeof test.controls != 'undefined' ){
			return true;
//console.info( test.id+' safe (control)');
		}else if( typeof test.modify != 'undefined' ){
			return true;
		}else if( typeof test.controlledBy == 'undefined' ){
			switch( test.type ){
				case 'yn':
					if( test.valueNo == true ){
						return true;
					}
					break;
				case 'bool':
					if( test.valueBool == null ||
						test.valueBool == 0  ){
						//console.info( test.id+' safe');
						return true;
					}
					break;
				case 'avpu':
					if( test.valueString == test.transfer ){
						//console.info( test.id+' safe');
						return true;
					}
					break;
				case 'number':
					var hasLower=false;
					if( typeof test.transferMin != 'undefined' ){
						hasLower=true;
					}
					var hasUpper=false;
					if( typeof test.transferMax != 'undefined' ){
						hasUpper=true;
					}

					if(
						( !hasLower || (hasLower && test.value > test.transferMin ) )
						&&
						( !hasUpper || (hasUpper && test.value < test.transferMax ) )
					){
//console.info( '_testSafe ' + test.id+' safe');
						return true;
					}
//console.info( '_testSafe ' + test.id+' break');
					break;
			}
		}else{
			var tests = $scope.steps[ 1 ].tests;//HACK, should pass in the step index or title
			for(var j=0;j<tests.length;j++){
				if( tests[j].id == test.controlledBy ){
					//console.debug( test.id+' controlled by '+tests[j].id );
					//if hidden, is valid
					//if shown, check value
					if( tests[j].valueBool != 1 ){
						//console.debug( test.id+' safe (hidden)' );
						return true;
					}else{
						var hasLower=false;
						if( typeof test.transferMin != 'undefined' ){
							hasLower=true;
						}
						var hasUpper=false;
						if( typeof test.transferMax != 'undefined' ){
							hasUpper=true;
						}

						if(
							( !hasLower || (hasLower && test.value > test.transferMin ) )
							&&
							( !hasUpper || (hasUpper && test.value < test.transferMax ) )
						){
							//console.info( test.id+' safe');
							return true;
						}

					}
				}
			}
		}
		//console.debug(test.id+' NOT safe');
		return false;
	}

	function _allSafe( steps ){
		var cnt=0;
		var length=0;

		for(var j=0;j<steps.length;j++){
			var test=steps[j].tests;
			length+=test.length;

			for(var i=0;i<test.length;i++){
				console.debug( test[i].id+' ?' );
				if( _testSafe(test[i]) ){
				console.debug( ' safe' );
					cnt++;
				}
			}
		}
		console.info('all safe : '+cnt+'/'+length+' are safe');
		return cnt==length
	}
})

.controller('ReferController', function($scope,$state,$ionicLoading,$ionicPopup, syncableDatabase,geoService,createEmailDialog,emailService,testService,testPrefDialog,wasLastResultTest,QuestionsController) {

	$scope.isCalcTest = wasLastResultTest;

	$scope.$on('$ionicView.enter',function(){
		testService.wasLastResultTest().then(function(res){$scope.isCalcTest=res;});
		if(  $state.params.testResults_id == -1 ){
			testPrefDialog();
		}
	});

	$scope.showTestInfo = function(evt){
		testService.showDialog(evt,$scope.isCalcTest,{
	      success:  function(newValue){
	        $scope.isCalcTest = newValue;
	      }
	    });
	}

	$scope.criteria = QuestionsController.getQuestions('criteria');

	$scope.anyYes=false;

	if( $state.params.testResults_id > -1 ){
		$scope.criteria[0].value = ($state.params.score == 3);
		$scope.anyYes            = ($state.params.score == 3);
	}

	$scope.$on('event:clearRefer', function(e, rejection) {
		for(var i=0;i<$scope.criteria.length;i++){
			$scope.criteria[i].value=false;
		}
		if ($scope.form) {
	      // reset validation's state
	      $scope.form.$setUntouched();
	    }
	    $scope.anyYes=false;
	});

	$scope.shouldShowMessage=function(){
		if( ! $scope.criteria ){
			return false;
		}
		if( $scope.anyYes ){
			return false;
		}
		return true;

	}
	$scope.shouldShowEmailAssessment=function(){
		if( $scope.anyYes ){
			return false;
		}
		if( $state.params.testResults_id > -1 ){
			return ( $state.params.score == 1 || $state.params.score == 2 ) ;
		}
		return false;
	}

	$scope.sendEmail=function(evt,refer_id){
		createEmailDialog(evt,{
          success:  function(email){
            $ionicLoading.show({
              template: 'Sending email...',
              cssClass: 'review'
            });

			syncableDatabase.db().transaction( function(tx) {
	    		var cri={};

	    		for(var i=0;i<$scope.criteria.length;i++){
					var t=$scope.criteria[i];
					cri[t.id] = t.value ;
		        }

				var data={
					criteria:cri
				}

	            var testResultId=$state.params.testResults_id ;
				tx.executeSql("select * from testResults where id=? order by date desc",[ testResultId ],function(tx,testResultsResults){
					tx.executeSql("select * from riskAssesment where testResults_id=? order by date desc",[ testResultId ],function(tx,riskAssesmentResults){
				    		tx.executeSql("select * from treatment where testResults_id=? order by date desc",[ testResultId ],function(tx,treatmentResults){

				            	data.calculation={
									score         : testResultsResults.rows.item(0)
								};
								data.emergencies=syncableDatabase.resultRowToStruct( riskAssesmentResults.rows.item(0) );
								data.assessment=syncableDatabase.resultRowToStruct( treatmentResults.rows.item(0) );

								emailService.sendEmail('referalResult_3',email,data)
								.then(function(data,status){
									$ionicLoading.hide();
									if(data.ok == true ){
									  // - set email done flag
									  $scope._saveEmail(email,true,refer_id);
									  $ionicPopup.alert({
									    title: 'Email sent',
									    template: 'The email will be with you shortly',
									    cssClass: 'refer ok',
										okType: 'button-balanced'
									  });
									}else{
									  console.error("remote error "+data.message);
									  //as error()
									  $ionicPopup.alert({
									    title: 'Email sending delay',
									    template: 'The email will be with you after a short delay',
									    cssClass: 'refer warn',
										okType: 'button-balanced'
									  });
									  $scope._saveEmail(email,false,refer_id);
									}
								})
								.catch(function(data,status){
									$ionicLoading.hide();
									console.error("failed to call "+data);
									$ionicPopup.alert({
										title: 'Could not find server' ,
										template: 'The email will be sent next time you go online.'+data,
										cssClass: 'refer error',
										okType: 'button-balanced'
									});
									$scope._saveEmail(email,false,refer_id);
								});
			            })
			        })
			    });//end transaction() for via calc
			}); //end transaction() for direct access refer queries
		}});//end createEmailDialog success handler
	}//end $scope.sendEmail

	$scope._saveEmail=function(email,didSend,refer_id){
	    syncableDatabase.db().transaction(function(tx) {
			tx.executeSql(
				'INSERT INTO referEmail ( date, refer_id ,emailTo, emailDone) values (?,?,?,?)',
				[
					new Date().getTime(),
					refer_id,
					email,
					didSend
				],
				function(){ syncableDatabase.sync(); },
				function(tx,e){console.error('INSERT INTO referEmail !ok'+e.message)}
			);
		});
	}

	$scope.continue = function(e){
	    syncableDatabase.db().transaction(function(tx) {
	    	//duplicated in _saveEmail()
			var params=[new Date().getTime(),$state.params.testResults_id,$scope.isCalcTest==true?1:0 ];

	    	var sql='insert into refer (date,testResults_id,isTest';

			for(var i=0;i<$scope.criteria.length;i++){
				var t=$scope.criteria[i];
				sql+=','+t.id;
				params=params.concat( t.value );
	        }

			sql+=') values (';

			var valueSql='';//one '?,' for every entry in params
			for(var i=0;i < params.length;i++){
				if( i == params.length-1 ){
				  sql+='?';
				}else{
				  sql+='?,';
				}
			}

			sql+=')';

			tx.executeSql(
				sql,params,
				function(){
					syncableDatabase.sync();

					if( e ){ //came from the email assessement button
						syncableDatabase.db().transaction(function(tx) {
							tx.executeSql('select max(id) as id from refer',[], function (tx,results){
								$scope.sendEmail( e,results.rows.item(0).id );
							});
						});
					}else if( $scope.anyYes ){
						syncableDatabase.db().transaction(function(tx) {
							tx.executeSql('select max(id) as id from refer',[], function (tx,results){
								$state.go("app.refer-safety",{
									refer_id       : results.rows.item(0).id,
									testResults_id : $state.params.testResults_id
								});
							});
						});
					}else{
						$state.go('app.summary')
					}
				},
				function(t,e){console.error('failed to save');console.dir(e);}
			);
		});
	}

	$scope.updateAnyYes=function(){
		$scope.anyYes=_anyYes( $scope.criteria );
	}

	function _anyYes( criteria ){
		if( typeof criteria == 'undefined' ) return false;

		for(var i=0;i<criteria.length;i++){
			if( criteria[i].value == true ){
				return true;
			}
		}
		return false;
	}
});
