"use strict";
angular.module("geo-service",[]).
factory('geoService',function($rootScope,$q){

$rootScope.geoServiceHasPosition=false;

// http://www.movable-type.co.uk/scripts/latlong.html
function distance(lat1,lon1,lat2,lon2){
	var R = 6373; // kilometers
	var d1 = radians(lat1);
	var d2 = radians(lat2);
	var aa = radians(lat2-lat1);
	var ab = radians(lon2-lon1);

	var a = Math.sin(aa/2) * Math.sin(aa/2) +
	        Math.cos(d1) * Math.cos(d2) *
	        Math.sin(ab/2) * Math.sin(ab/2);
	var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));

	return R * c;
}

function radians (degrees) {
  return degrees * Math.PI / 180;
}

return {
	update:function(){
		navigator.geolocation.getCurrentPosition(onCurrentPositionSuccess, onCurrentPositionError, { maximumAge: 3000, timeout: 5000} );

		console.debug('geoService : update... ');

		function onCurrentPositionSuccess (position) {
			var element = document.getElementById('geolocation');
			$rootScope.geoServicelocationDebug = 'Latitude: '          + position.coords.latitude         + ',' +
			                    'Longitude: '         + position.coords.longitude        + ',' +
			                    'Altitude: '          + position.coords.altitude         + ',' +
			                    'Accuracy: '          + position.coords.accuracy         + ',' +
			                    'Altitude Accuracy: ' + position.coords.altitudeAccuracy + ',' +
			                    'Heading: '           + position.coords.heading          + ',' +
			                    'Speed: '             + position.coords.speed            + ',' +
			                    'Timestamp: '         + position.timestamp               ;

			$rootScope.geoServiceLatitude    = position.coords.latitude;
			$rootScope.geoServiceLongitude   = position.coords.longitude;
			$rootScope.geoServiceHasPosition = true;
			console.debug('geoService : '+$rootScope.geoServicelocationDebug);
		}
		function onCurrentPositionError(error) {
			$rootScope.geoServicelocationDebug = 'error : '    + error.code    + ' message: ' + error.message + '\n';
			console.debug('geoService : '+$rootScope.geoServicelocationDebug);
		}
	},

	getLocationDebug : function(){return $rootScope.geoServicelocationDebug;},

	hasPosition: function(){
		return $rootScope.geoServiceHasPosition;
	},

	getPosition: function(){
		return {
			latitude  : $rootScope.geoServiceLatitude,
			longitude : $rootScope.geoServiceLongitude,
		}
	},

	get: function(geolocationOptions){
		var deferred = $q.defer();

		geolocationOptions = typeof geolocationOptions !== 'undefined' ? geolocationOptions : {maximumAge: 30*60*1000, timeout: 5000};

		navigator.geolocation.getCurrentPosition(
			function onCurrentPositionSuccess (position) {
				$rootScope.geoServiceLatitude    = position.coords.latitude;
				$rootScope.geoServiceLongitude   = position.coords.longitude;
				$rootScope.geoServiceHasPosition = true;
				$rootScope.geoServicelocationDebug =
								'Latitude: '          + position.coords.latitude         + ',' +
			                    'Longitude: '         + position.coords.longitude        + ',' +
			                    'Altitude: '          + position.coords.altitude         + ',' +
			                    'Accuracy: '          + position.coords.accuracy         + ',' +
			                    'Altitude Accuracy: ' + position.coords.altitudeAccuracy + ',' +
			                    'Heading: '           + position.coords.heading          + ',' +
			                    'Speed: '             + position.coords.speed            + ',' +
			                    'Timestamp: '         + position.timestamp               + ',' +
			                    'Date: '              + new Date( position.timestamp )   ;
				if(console&&console.debug) console.debug('geoService async : '+$rootScope.geoServicelocationDebug);
				deferred.resolve(position);
			},
			function onCurrentPositionError(error) {
				if(console&&console.error) console.error(error);

				$rootScope.geoServicelocationDebug = 'async error : '    + error.code    + ' message: ' + error.message + '\n';
				if(console&&console.debug) console.debug('geoService : '+$rootScope.geoServicelocationDebug);

				deferred.reject(error);
			},
			geolocationOptions
		);

		return deferred.promise;
	},

	getRegions : function(){
		return [
			{
				name : 'Greater Manchester, Lancashire and South Cumbria Strategic Clinical Network',
				hospitals : [
					{id:'CMFT',name:'Central Manchester NHS Foundation Trust',latitude:53.46293,longitude:-2.22640,tel:'0161 276 1234',locations:['Stockport (Stepping Hill)','Wythenshawe','Tameside','North Manchester Hospital' ]},
					{id:'SRFT',name:'Salford Royal NHS Foundation Trust',latitude:53.48752,longitude: -2.32344,tel:'0161 789 7373',locations:['Bolton','Oldham','Bury','Trafford','Fairfield','Wigan',]},
					{id:'LTHTR',name:'Royal Preston Hospital',latitude:53.791449,longitude: -2.709159,tel:'01772 716 565',locations:['Blackburn','Burnley','Barrow', 'Lancaster','Kendal','Chorley','Blackpool']}
				],
				postCodeData : [ { postcode:'BB1', hospitalCode:'LTHTR', lat:53.75652, lon:-2.46335 } , { postcode:'BB10', hospitalCode:'LTHTR', lat:53.79946, lon:-2.21978 } , { postcode:'BB11', hospitalCode:'LTHTR', lat:53.78285, lon:-2.25353 } , { postcode:'BB12', hospitalCode:'LTHTR', lat:53.80355, lon:-2.29251 } , { postcode:'BB18', hospitalCode:'LTHTR', lat:53.91457, lon:-2.17192 } , { postcode:'BB2', hospitalCode:'LTHTR', lat:53.74171, lon:-2.50154 } , { postcode:'BB3', hospitalCode:'LTHTR', lat:53.69811, lon:-2.46666 } , { postcode:'BB4', hospitalCode:'LTHTR', lat:53.70379, lon:-2.29191 } , { postcode:'BB5', hospitalCode:'LTHTR', lat:53.75425, lon:-2.37317 } , { postcode:'BB6', hospitalCode:'LTHTR', lat:53.79292, lon:-2.41789 } , { postcode:'BB7', hospitalCode:'LTHTR', lat:53.87281, lon:-2.38796 } , { postcode:'BB8', hospitalCode:'LTHTR', lat:53.85800, lon:-2.16340 } , { postcode:'BB9', hospitalCode:'LTHTR', lat:53.83739, lon:-2.21568 } , { postcode:'BL0', hospitalCode:'SRFT', lat:53.64854, lon:-2.31704 } , { postcode:'BL1', hospitalCode:'SRFT', lat:53.59037, lon:-2.44462 } , { postcode:'BL2', hospitalCode:'SRFT', lat:53.58847, lon:-2.39512 } , { postcode:'BL3', hospitalCode:'SRFT', lat:53.56483, lon:-2.43225 } , { postcode:'BL4', hospitalCode:'SRFT', lat:53.54652, lon:-2.40246 } , { postcode:'BL5', hospitalCode:'SRFT', lat:53.54835, lon:-2.51614 } , { postcode:'BL6', hospitalCode:'SRFT', lat:53.59176, lon:-2.54037 } , { postcode:'BL7', hospitalCode:'SRFT', lat:53.63151, lon:-2.42231 } , { postcode:'BL8', hospitalCode:'SRFT', lat:53.60463, lon:-2.32731 } , { postcode:'BL9', hospitalCode:'SRFT', lat:53.59195, lon:-2.28716 } , { postcode:'FY1', hospitalCode:'LTHTR', lat:53.81531, lon:-3.04708 } , { postcode:'FY2', hospitalCode:'LTHTR', lat:53.84574, lon:-3.03987 } , { postcode:'FY3', hospitalCode:'LTHTR', lat:53.82119, lon:-3.02055 } , { postcode:'FY4', hospitalCode:'LTHTR', lat:53.79130, lon:-3.02858 } , { postcode:'FY5', hospitalCode:'LTHTR', lat:53.87529, lon:-3.02244 } , { postcode:'FY6', hospitalCode:'LTHTR', lat:53.86848, lon:-2.98217 } , { postcode:'FY7', hospitalCode:'LTHTR', lat:53.91591, lon:-3.02767 } , { postcode:'FY8', hospitalCode:'LTHTR', lat:53.75044, lon:-3.00222 } , { postcode:'LA1', hospitalCode:'LTHTR', lat:54.04654, lon:-2.79970 } , { postcode:'LA10', hospitalCode:'LTHTR', lat:54.31416, lon:-2.49454 } , { postcode:'LA11', hospitalCode:'LTHTR', lat:54.20033, lon:-2.92582 } , { postcode:'LA12', hospitalCode:'LTHTR', lat:54.20324, lon:-3.08371 } , { postcode:'LA13', hospitalCode:'LTHTR', lat:54.11815, lon:-3.19827 } , { postcode:'LA14', hospitalCode:'LTHTR', lat:54.11821, lon:-3.23127 } , { postcode:'LA15', hospitalCode:'LTHTR', lat:54.15578, lon:-3.17963 } , { postcode:'LA16', hospitalCode:'LTHTR', lat:54.18758, lon:-3.20293 } , { postcode:'LA17', hospitalCode:'LTHTR', lat:54.23950, lon:-3.17593 } , { postcode:'LA18', hospitalCode:'LTHTR', lat:54.21552, lon:-3.27605 } , { postcode:'LA19', hospitalCode:'LTHTR', lat:54.29699, lon:-3.37999 } , { postcode:'LA2', hospitalCode:'LTHTR', lat:54.06656, lon:-2.67096 } , { postcode:'LA20', hospitalCode:'LTHTR', lat:54.29403, lon:-3.20638 } , { postcode:'LA21', hospitalCode:'LTHTR', lat:54.36333, lon:-3.07806 } , { postcode:'LA22', hospitalCode:'LTHTR', lat:54.42272, lon:-2.99072 } , { postcode:'LA23', hospitalCode:'LTHTR', lat:54.37177, lon:-2.91337 } , { postcode:'LA3', hospitalCode:'LTHTR', lat:54.04992, lon:-2.87948 } , { postcode:'LA4', hospitalCode:'LTHTR', lat:54.07023, lon:-2.85268 } , { postcode:'LA5', hospitalCode:'LTHTR', lat:54.14387, lon:-2.79162 } , { postcode:'LA6', hospitalCode:'LTHTR', lat:54.17520, lon:-2.61134 } , { postcode:'LA7', hospitalCode:'LTHTR', lat:54.22568, lon:-2.77319 } , { postcode:'LA8', hospitalCode:'LTHTR', lat:54.32384, lon:-2.76127 } , { postcode:'LA9', hospitalCode:'LTHTR', lat:54.32572, lon:-2.74351 } , { postcode:'M11', hospitalCode:'CMFT', lat:53.47902, lon:-2.18183 } , { postcode:'M12', hospitalCode:'CMFT', lat:53.46433, lon:-2.20246 } , { postcode:'M13', hospitalCode:'CMFT', lat:53.46075, lon:-2.21591 } , { postcode:'M14', hospitalCode:'CMFT', lat:53.44819, lon:-2.22567 } , { postcode:'M15', hospitalCode:'CMFT', lat:53.46683, lon:-2.25166 } , { postcode:'M16', hospitalCode:'CMFT', lat:53.45511, lon:-2.26448 } , { postcode:'M17', hospitalCode:'SRFT', lat:53.46829, lon:-2.32054 } , { postcode:'M17', hospitalCode:'CMFT', lat:53.46829, lon:-2.32054 } , { postcode:'M18', hospitalCode:'CMFT', lat:53.46136, lon:-2.17098 } , { postcode:'M19', hospitalCode:'CMFT', lat:53.43643, lon:-2.19635 } , { postcode:'M2', hospitalCode:'CMFT', lat:53.48033, lon:-2.24455 } , { postcode:'M20', hospitalCode:'CMFT', lat:53.42449, lon:-2.23251 } , { postcode:'M21', hospitalCode:'CMFT', lat:53.43808, lon:-2.27288 } , { postcode:'M22', hospitalCode:'CMFT', lat:53.38599, lon:-2.25992 } , { postcode:'M23', hospitalCode:'CMFT', lat:53.39957, lon:-2.28889 } , { postcode:'M24', hospitalCode:'SRFT', lat:53.55114, lon:-2.19798 } , { postcode:'M25', hospitalCode:'SRFT', lat:53.52840, lon:-2.27576 } , { postcode:'M26', hospitalCode:'SRFT', lat:53.56202, lon:-2.33510 } , { postcode:'M27', hospitalCode:'SRFT', lat:53.51221, lon:-2.33680 } , { postcode:'M28', hospitalCode:'SRFT', lat:53.51455, lon:-2.39905 } , { postcode:'M29', hospitalCode:'SRFT', lat:53.50852, lon:-2.45860 } , { postcode:'M3', hospitalCode:'SRFT', lat:53.48348, lon:-2.25238 } , { postcode:'M3', hospitalCode:'CMFT', lat:53.48348, lon:-2.25238 } , { postcode:'M30', hospitalCode:'SRFT', lat:53.48441, lon:-2.35543 } , { postcode:'M31', hospitalCode:'SRFT', lat:53.41944, lon:-2.42386 } , { postcode:'M32', hospitalCode:'SRFT', lat:53.45074, lon:-2.30959 } , { postcode:'M33', hospitalCode:'CMFT', lat:53.42105, lon:-2.32612 } , { postcode:'M33 4', hospitalCode:'SRFT', lat:53.42105, lon:-2.32612 } , { postcode:'M33 5', hospitalCode:'SRFT', lat:53.42105, lon:-2.32612 } , { postcode:'M33 6', hospitalCode:'SRFT', lat:53.42105, lon:-2.32612 } , { postcode:'M33 7', hospitalCode:'SRFT', lat:53.42105, lon:-2.32612 } , { postcode:'M34', hospitalCode:'CMFT', lat:53.45638, lon:-2.11861 } , { postcode:'M35', hospitalCode:'SRFT', lat:53.50729, lon:-2.15460 } , { postcode:'M38', hospitalCode:'SRFT', lat:53.53225, lon:-2.42292 } , { postcode:'M4', hospitalCode:'CMFT', lat:53.48502, lon:-2.23056 } , { postcode:'M40', hospitalCode:'CMFT', lat:53.50424, lon:-2.19083 } , { postcode:'M41', hospitalCode:'SRFT', lat:53.45101, lon:-2.36522 } , { postcode:'M43', hospitalCode:'CMFT', lat:53.48380, lon:-2.14934 } , { postcode:'M44', hospitalCode:'SRFT', lat:53.44077, lon:-2.42722 } , { postcode:'M45', hospitalCode:'SRFT', lat:53.54734, lon:-2.29037 } , { postcode:'M46', hospitalCode:'SRFT', lat:53.52580, lon:-2.49202 } , { postcode:'M5', hospitalCode:'SRFT', lat:53.47928, lon:-2.28551 } , { postcode:'M50', hospitalCode:'SRFT', lat:53.47648, lon:-2.29348 } , { postcode:'M6', hospitalCode:'SRFT', lat:53.49261, lon:-2.29925 } , { postcode:'M7', hospitalCode:'SRFT', lat:53.50560, lon:-2.26238 } , { postcode:'M8', hospitalCode:'CMFT', lat:53.50872, lon:-2.24024 } , { postcode:'M9', hospitalCode:'CMFT', lat:53.52202, lon:-2.21407 } , { postcode:'M90', hospitalCode:'CMFT', lat:53.36498, lon:-2.27729 } , { postcode:'OL1', hospitalCode:'SRFT', lat:53.54951, lon:-2.10682 } , { postcode:'OL10', hospitalCode:'SRFT', lat:53.59057, lon:-2.22300 } , { postcode:'OL11', hospitalCode:'LTHTR', lat:53.60599, lon:-2.17695 } , { postcode:'OL11', hospitalCode:'SRFT', lat:53.60599, lon:-2.17695 } , { postcode:'OL12', hospitalCode:'SRFT', lat:53.63634, lon:-2.16548 } , { postcode:'OL13', hospitalCode:'LTHTR', lat:53.70080, lon:-2.20450 } , { postcode:'OL14', hospitalCode:'LTHTR', lat:53.71453, lon:-2.10100 } , { postcode:'OL15', hospitalCode:'SRFT', lat:53.64462, lon:-2.10023 } , { postcode:'OL16', hospitalCode:'SRFT', lat:53.61263, lon:-2.13197 } , { postcode:'OL2', hospitalCode:'SRFT', lat:53.57317, lon:-2.10997 } , { postcode:'OL3', hospitalCode:'SRFT', lat:53.55358, lon:-2.01092 } , { postcode:'OL4', hospitalCode:'SRFT', lat:53.54229, lon:-2.07364 } , { postcode:'OL5', hospitalCode:'CMFT', lat:53.51761, lon:-2.03911 } , { postcode:'OL6', hospitalCode:'CMFT', lat:53.49409, lon:-2.08659 } , { postcode:'OL7', hospitalCode:'CMFT', lat:53.49064, lon:-2.10714 } , { postcode:'OL8', hospitalCode:'SRFT', lat:53.52592, lon:-2.11708 } , { postcode:'OL9', hospitalCode:'SRFT', lat:53.53924, lon:-2.14405 } , { postcode:'PR1', hospitalCode:'LTHTR', lat:53.75743, lon:-2.70204 } , { postcode:'PR2', hospitalCode:'LTHTR', lat:53.77765, lon:-2.70752 } , { postcode:'PR25', hospitalCode:'LTHTR', lat:53.69441, lon:-2.69436 } , { postcode:'PR26', hospitalCode:'LTHTR', lat:53.68747, lon:-2.74150 } , { postcode:'PR3', hospitalCode:'LTHTR', lat:53.86953, lon:-2.71878 } , { postcode:'PR4', hospitalCode:'LTHTR', lat:53.75260, lon:-2.83488 } , { postcode:'PR5', hospitalCode:'LTHTR', lat:53.73156, lon:-2.65777 } , { postcode:'PR6', hospitalCode:'SRFT', lat:53.66763, lon:-2.61213 } , { postcode:'PR7', hospitalCode:'LTHTR', lat:53.64812, lon:-2.65450 } , { postcode:'PR9', hospitalCode:'LTHTR', lat:53.65902, lon:-2.96957 } , { postcode:'SK1', hospitalCode:'CMFT', lat:53.40772, lon:-2.15176 } , { postcode:'SK13', hospitalCode:'CMFT', lat:53.44859, lon:-1.96506 } , { postcode:'SK14', hospitalCode:'CMFT', lat:53.45198, lon:-2.05660 } , { postcode:'SK15', hospitalCode:'CMFT', lat:53.48696, lon:-2.04722 } , { postcode:'SK16', hospitalCode:'CMFT', lat:53.47422, lon:-2.08260 } , { postcode:'SK2', hospitalCode:'CMFT', lat:53.39437, lon:-2.13260 } , { postcode:'SK3', hospitalCode:'CMFT', lat:53.39829, lon:-2.17214 } , { postcode:'SK4', hospitalCode:'CMFT', lat:53.41923, lon:-2.18134 } , { postcode:'SK5', hospitalCode:'CMFT', lat:53.43523, lon:-2.15277 } , { postcode:'SK6', hospitalCode:'CMFT', lat:53.40534, lon:-2.08052 } , { postcode:'SK7', hospitalCode:'CMFT', lat:53.37019, lon:-2.14367 } , { postcode:'SK8', hospitalCode:'CMFT', lat:53.37984, lon:-2.20921 } , { postcode:'SK9', hospitalCode:'CMFT', lat:53.32765, lon:-2.23089 } , { postcode:'WA13', hospitalCode:'CMFT', lat:53.38242, lon:-2.46684 } , { postcode:'WA14', hospitalCode:'CMFT', lat:53.38765, lon:-2.36038 } , { postcode:'WA15', hospitalCode:'CMFT', lat:53.38439, lon:-2.32611 } , { postcode:'WA3', hospitalCode:'SRFT', lat:53.45591, lon:-2.54753 } , { postcode:'WN1', hospitalCode:'SRFT', lat:53.55320, lon:-2.62838 } , { postcode:'WN2', hospitalCode:'SRFT', lat:53.53604, lon:-2.58232 } , { postcode:'WN3', hospitalCode:'SRFT', lat:53.52890, lon:-2.64579 } , { postcode:'WN4', hospitalCode:'SRFT', lat:53.49387, lon:-2.64432 } , { postcode:'WN5', hospitalCode:'SRFT', lat:53.53193, lon:-2.68723 } , { postcode:'WN6', hospitalCode:'SRFT', lat:53.57482, lon:-2.67211 } , { postcode:'WN7', hospitalCode:'SRFT', lat:53.49806, lon:-2.51792 } , { postcode:'WN8', hospitalCode:'SRFT', lat:53.55291, lon:-2.76959 } ]
			},
			{
				name : 'North West Coast Strategic Clinical Network',
				hospitals:[
					{id:'RLBUHT',name:'Royal Liverpool & Broadgreen Univeristy Hospitals NHS Trust',latitude:53.4095,longitude:-2.9648,tel:'0151 706 2000',locations:['Liverpool (Heart & Chest)','Liverpool (Women\'s)','Warrington','Halton','Whiston & St Helens','Nobles (Isle of Man)']},
					{id:'AFT',name:'Aintree Foundation Trust',latitude:53.4679181,longitude:-2.9351308,tel:'0151 525 5980',locations:['Southport','Ormskirk','Walton Centre','Merseycare']},
					{id:'WUH',name:'Wirral Univeristy hospitals Foundation Trust',latitude:53.3708,longitude:-3.0954,tel:'0151 678 5111',locations:['Arrowe Park','Clatterbridge','Countess of Chester']}
				],
				postCodeData:[ { postcode:'CH1', hospitalCode:'RLBUHT', lat:53.20210, lon:-2.90915 } , { postcode:'CH2', hospitalCode:'RLBUHT', lat:53.21784, lon:-2.86899 } , { postcode:'CH3', hospitalCode:'RLBUHT', lat:53.16545, lon:-2.81939 } , { postcode:'CH4', hospitalCode:'RLBUHT', lat:53.16619, lon:-2.94626 } , { postcode:'CW1', hospitalCode:'RLBUHT', lat:53.10282, lon:-2.43601 } , { postcode:'CW10', hospitalCode:'RLBUHT', lat:53.18915, lon:-2.44375 } , { postcode:'CW11', hospitalCode:'RLBUHT', lat:53.14294, lon:-2.36708 } , { postcode:'CW12', hospitalCode:'RLBUHT', lat:53.16535, lon:-2.21269 } , { postcode:'CW2', hospitalCode:'RLBUHT', lat:53.08042, lon:-2.44608 } , { postcode:'CW3', hospitalCode:'RLBUHT', lat:52.99826, lon:-2.40630 } , { postcode:'CW4', hospitalCode:'RLBUHT', lat:53.20861, lon:-2.35355 } , { postcode:'CW5', hospitalCode:'RLBUHT', lat:53.06187, lon:-2.52161 } , { postcode:'CW6', hospitalCode:'RLBUHT', lat:53.16249, lon:-2.66736 } , { postcode:'CW7', hospitalCode:'RLBUHT', lat:53.19021, lon:-2.52872 } , { postcode:'CW8', hospitalCode:'RLBUHT', lat:53.25350, lon:-2.56518 } , { postcode:'CW9', hospitalCode:'RLBUHT', lat:53.25924, lon:-2.50246 } , { postcode:'SK10', hospitalCode:'RLBUHT', lat:53.27768, lon:-2.13050 } , { postcode:'SK11', hospitalCode:'RLBUHT', lat:53.24534, lon:-2.14489 } , { postcode:'SK12', hospitalCode:'RLBUHT', lat:53.35160, lon:-2.09299 } , { postcode:'SK9', hospitalCode:'RLBUHT', lat:53.32765, lon:-2.23089 } , { postcode:'ST7', hospitalCode:'RLBUHT', lat:53.08835, lon:-2.26675 } , { postcode:'SY13', hospitalCode:'RLBUHT', lat:52.95240, lon:-2.69026 } , { postcode:'SY14', hospitalCode:'RLBUHT', lat:53.03134, lon:-2.76753 } , { postcode:'WA1', hospitalCode:'RLBUHT', lat:53.39412, lon:-2.57229 } , { postcode:'WA13', hospitalCode:'RLBUHT', lat:53.38241, lon:-2.46733 } , { postcode:'WA14', hospitalCode:'RLBUHT', lat:53.38765, lon:-2.36034 } , { postcode:'WA15', hospitalCode:'RLBUHT', lat:53.38438, lon:-2.32603 } , { postcode:'WA16', hospitalCode:'RLBUHT', lat:53.30322, lon:-2.37176 } , { postcode:'WA2', hospitalCode:'RLBUHT', lat:53.41066, lon:-2.58137 } , { postcode:'WA3', hospitalCode:'RLBUHT', lat:53.45591, lon:-2.54745 } , { postcode:'WA4', hospitalCode:'RLBUHT', lat:53.36582, lon:-2.57645 } , { postcode:'WA5', hospitalCode:'RLBUHT', lat:53.39874, lon:-2.63806 } , { postcode:'WA6', hospitalCode:'RLBUHT', lat:53.27276, lon:-2.72426 } , { postcode:'WA7', hospitalCode:'RLBUHT', lat:53.33030, lon:-2.70411 } , { postcode:'WA8', hospitalCode:'RLBUHT', lat:53.37290, lon:-2.74182 } , { postcode:'CH41', hospitalCode:'RLBUHT', lat:53.39413, lon:-3.03228 } , { postcode:'CH42', hospitalCode:'RLBUHT', lat:53.37405, lon:-3.02285 } , { postcode:'CH43', hospitalCode:'RLBUHT', lat:53.38422, lon:-3.05962 } , { postcode:'CH44', hospitalCode:'RLBUHT', lat:53.41529, lon:-3.03710 } , { postcode:'CH45', hospitalCode:'RLBUHT', lat:53.42885, lon:-3.05164 } , { postcode:'CH46', hospitalCode:'RLBUHT', lat:53.40401, lon:-3.11091 } , { postcode:'CH47', hospitalCode:'RLBUHT', lat:53.39640, lon:-3.16932 } , { postcode:'CH48', hospitalCode:'RLBUHT', lat:53.37041, lon:-3.16944 } , { postcode:'CH49', hospitalCode:'RLBUHT', lat:53.38099, lon:-3.10395 } , { postcode:'CH60', hospitalCode:'RLBUHT', lat:53.32653, lon:-3.09608 } , { postcode:'CH61', hospitalCode:'RLBUHT', lat:53.34905, lon:-3.10199 } , { postcode:'CH63', hospitalCode:'RLBUHT', lat:53.34438, lon:-3.01237 } , { postcode:'L10', hospitalCode:'RLBUHT', lat:53.47450, lon:-2.92690 } , { postcode:'L12', hospitalCode:'RLBUHT', lat:53.43504, lon:-2.89566 } , { postcode:'L14', hospitalCode:'RLBUHT', lat:53.41799, lon:-2.88100 } , { postcode:'L16', hospitalCode:'RLBUHT', lat:53.39864, lon:-2.88846 } , { postcode:'L17', hospitalCode:'RLBUHT', lat:53.37759, lon:-2.94054 } , { postcode:'L18', hospitalCode:'RLBUHT', lat:53.38057, lon:-2.90845 } , { postcode:'L19', hospitalCode:'RLBUHT', lat:53.35896, lon:-2.90336 } , { postcode:'L20', hospitalCode:'RLBUHT', lat:53.45296, lon:-2.98921 } , { postcode:'L21', hospitalCode:'RLBUHT', lat:53.47140, lon:-3.00051 } , { postcode:'L22', hospitalCode:'RLBUHT', lat:53.47726, lon:-3.02791 } , { postcode:'L23', hospitalCode:'RLBUHT', lat:53.49105, lon:-3.02391 } , { postcode:'L24', hospitalCode:'RLBUHT', lat:53.34364, lon:-2.83866 } , { postcode:'L25', hospitalCode:'RLBUHT', lat:53.37879, lon:-2.86355 } , { postcode:'L26', hospitalCode:'RLBUHT', lat:53.36441, lon:-2.83299 } , { postcode:'L29', hospitalCode:'RLBUHT', lat:53.50765, lon:-2.98538 } , { postcode:'L3', hospitalCode:'RLBUHT', lat:53.40845, lon:-2.98449 } , { postcode:'L30', hospitalCode:'RLBUHT', lat:53.48406, lon:-2.97245 } , { postcode:'L31', hospitalCode:'RLBUHT', lat:53.51346, lon:-2.93813 } , { postcode:'L32', hospitalCode:'RLBUHT', lat:53.47822, lon:-2.88935 } , { postcode:'L34', hospitalCode:'RLBUHT', lat:53.43618, lon:-2.81558 } , { postcode:'L35', hospitalCode:'RLBUHT', lat:53.41509, lon:-2.78289 } , { postcode:'L36', hospitalCode:'RLBUHT', lat:53.41443, lon:-2.84151 } , { postcode:'L37', hospitalCode:'RLBUHT', lat:53.55712, lon:-3.06372 } , { postcode:'L38', hospitalCode:'RLBUHT', lat:53.52459, lon:-3.05150 } , { postcode:'L9', hospitalCode:'RLBUHT', lat:53.46151, lon:-2.95328 } , { postcode:'PR8', hospitalCode:'RLBUHT', lat:53.62617, lon:-3.00997 } , { postcode:'PR9', hospitalCode:'RLBUHT', lat:53.65913, lon:-2.96927 } , { postcode:'WA10', hospitalCode:'RLBUHT', lat:53.45418, lon:-2.75605 } , { postcode:'WA11', hospitalCode:'RLBUHT', lat:53.47802, lon:-2.71986 } , { postcode:'WA12', hospitalCode:'RLBUHT', lat:53.45369, lon:-2.63283 } , { postcode:'WA8', hospitalCode:'RLBUHT', lat:53.37290, lon:-2.74182 } , { postcode:'WA9', hospitalCode:'RLBUHT', lat:53.43620, lon:-2.71914 } , { postcode:'WN4', hospitalCode:'RLBUHT', lat:53.49387, lon:-2.64432 } , { postcode:'WN5', hospitalCode:'RLBUHT', lat:53.53193, lon:-2.68723 } ]
			}
		]
	},

	getRegionForPosition : function( position ){
		var pos=0;
		var minDistance=Number.MAX_VALUE;
		var regions = this.getRegions();
		var selected = this.getRegions()[0];

		for( var regionIdx=0;regionIdx<regions.length;regionIdx++ ){
			var region = regions[regionIdx];
			for( var hospitalIdx=0;hospitalIdx<region.hospitals.length;hospitalIdx++ ){
				var hospital = regions[regionIdx].hospitals[hospitalIdx];

				var d = distance(
					position.latitude,position.longitude,
					hospital.lat,hospital.long
				);
				if(d<minDistance){
					minDistance=d;
					pos=i;
					selected = regions[regionIdx].hospitals[hospitalIdx];
				}
			}
console.debug('geoService getRegionForPosition : @ '+region.name+' postcode search. Match '+pos+' @ '+minDistance+'km');
		}
console.info('geoService getRegionForPosition : '+selected.name);

		return selected;
	},

	setCurrentRegion : function( r ){
		this.currentRegion = r;
	},

	getCurrentRegion : function(){
		return this.currentRegion;
	},

	distance : distance

}

});
