"use strict";
angular.module("history-service",[])
.factory('historyService', function($q,syncableDatabase,QuestionsController) {

  var sortHistory = function(results) {
    var res = []
    for (var i = 0; i < results.rows.length; i++) {
      var row = results.rows.item(i)
      if (row.testResults_id == -1) {
        row.from = 'refer'
        if (row.aki == 'true') {
          row.score = 3
        }
      } else {
        row.from = 'testResults'
      }
      res.push(row)
    }
    return res
  }

  var thenTR = function(resTR){
    var row = resTR.rows.item(0);
    var detail = {
      id      : row.id,
      date    : row.date,
      isTest  : row.isTest,
      type    : row.type,
      akiResult: [
        {title: 'AKI Score',  value:  row.score}
      ]
    };
    if ( row.type == '' || row.type == 'creatinine' ) {
      baseline = row.baseline + ' µmol/L';
      current  = row.current + ' µmol/L';
      within48 = row.within48hours;
      if (resTR.rows.item(0).score == undefined) {
        detail.akiResult[0].value  = 3;
        var baseline = 'N/A';
        var current  = 'N/A';
        var within48 = 'N/A';
      }
      detail.akiResult.push({title: 'Baseline Creatinine', value: baseline});
      detail.akiResult.push({title: 'Current Creatinine' , value: current});
      detail.akiResult.push({title: 'Within 48 hours', value: within48});
    }
    if( row.type == '' || row.type == 'urine' ){
      if (row.urineRate == -1) {
        detail.akiResult.push({title: 'Urine Rate', value: row.urineRate});
      } else {
        detail.akiResult.push({title: 'Urine Rate', value: row.urineRate+' ml/kg/hr'});
      };
      if (row.urineHours == 'undefined') {
        detail.akiResult.push({title: 'Urine Hours' , value: row.urineHours});
      } else {
        detail.akiResult.push({title: 'Urine Hours' , value: row.urineHours+ ' hours'});
      };
    }
    return detail;
  }

  var thenRA = function(resRA){
    var results = [];
    if (resRA.rows.length > 0) {
      results = [
        {title: 'Potassium level', value: resRA.rows.item(0).pot, units: 'mmol/L'},
        {title: 'pH', value: resRA.rows.item(0).ph},
        {title: 'Acute confusion', value: resRA.rows.item(0).confusion},
        {title: 'Metabolic flap', value: resRA.rows.item(0).flap},
        {title: 'Pericardial rub', value: resRA.rows.item(0).rub},
        {title: 'Pulmonary oedema', value: resRA.rows.item(0).oedema}
      ]
    } else{
      results = [];
    };
    return results
  }

  var thenRisk = function(res){
    var results = [];
    if (res.rows.length > 0) {
      var list = QuestionsController.getQuestions('risk');
      for( var i=0;i < list.length;i++ ){
        for( var j=0;j < list[i].risks.length;j++ ){
          var value = list[i].risks[j].id;
          results.push({
            title : list[i].risks[j].label,
            value : res.rows.item(0)[value]
          });
        }
      }
    }
    return results;
  }

  var thenDyn = function(res,getList,additional){
    var list = QuestionsController.getQuestions(getList);
    var results = [];
    if (res.rows.length > 0) {
      if (additional == 'title') {
        for (var i = 0; i < list.length; i++) {
          results[i] = {
            title : list[i].title,
            tests : []
          }
          for (var j = 0; j < list[i]['tests'].length; j++) {
            var value = list[i]['tests'][j].id;
            results[i]['tests'][j] = {
              title : list[i]['tests'][j].label,
              value : res.rows.item(0)[value],
              units : list[i]['tests'][j].units
            }
          };
        };
      } else {
        for (var i = 0; i < list.length; i++) {
          var value = list[i].id;
          results[i] = {
            title : list[i].label,
            value : res.rows.item(0)[value]
          }
        };
      };
      if (getList == 'steps') {
        var listStepsSafe = {};
        listStepsSafe = {
          title : 'safe to transfer',
          value : res.rows.item(0).isSafe
        }
        results.push(listStepsSafe);
      }
    } else {
      results = [];
    };
    return results
  }

  return {
    getHistory: function() {
  		return $q.all([
        syncableDatabase.executeSql('select * from refer where testResults_id = -1 and isTest != 1')  .then(sortHistory),
        syncableDatabase.executeSql('select * from testResults where isTest != 1')                    .then(sortHistory)
  		]).then(function(res) {
  			return [].concat(res[0],res[1])
      }).catch(function(e){
  			throw e;
  		});
    },
    getDetail: function(table, id) {
      if (table == 'testResults') {
        return $q.all([
  	      syncableDatabase.executeSql('select * from testResults where id=?',[id])                    .then(thenTR),
  				syncableDatabase.executeSql('select * from testResultsFurther where testResults_id=?',[id]) .then(thenRisk),
  				syncableDatabase.executeSql('select * from refer where testResults_id=?',[id])              .then(function(res){ return thenDyn(res,'criteria')           }),
  				syncableDatabase.executeSql('select * from referSafety where testResults_id=?',[id])        .then(function(res){ return thenDyn(res,'steps','title')      }),
  				syncableDatabase.executeSql('select * from treatment where testResults_id=?',[id])          .then(function(res){ return thenDyn(res,'stepsTreat','title') }),
  				syncableDatabase.executeSql('select * from riskAssesment where testResults_id=?',[id])      .then(thenRA)
  			])
  			.then(function(res){
          return res
        }).catch(function(e){
    			throw e;
    		})
      } else if (table == 'refer') {
        return $q.all([
  	      syncableDatabase.executeSql('select * from refer where id=?',[id])                          .then(thenTR),
  				syncableDatabase.executeSql('select * from refer where id=?',[id])                          .then(function(res){ return thenDyn(res,'criteria')            }),
  				syncableDatabase.executeSql('select * from referSafety where refer_id=?',[id])              .then(function(res){ return thenDyn(res,'steps','title')       })
  			])
  			.then(function(res) {
          if (res[1][0].value == 'true') {
            res[0].akiResult[0].value = 3
          }
          return res
        }).catch(function(e){
    			throw e;
    		})
      }
    }
  }
})
