"use strict";
angular.module("sync-service", ['rt.debounce']).
factory('syncableDatabase',function($q,debounce){
	var webSqlDb = window.openDatabase('mydb', '1.0', 'my first database',5 * 1000000); //size in megabytes = 5

    var self = this;
	self.db = webSqlDb;
	self.sync=debounce(15*1000, function(){
    		console.info('syncableDatabase sync @ '+new Date());
	   		window.DBSYNC.syncNow(
		      function(message, percent, msgKey) {
		       //console.log('syncNow Progress ' +message+' '+percent+'%');
		      },
		      function(result) {
		        //console.log('syncNow result '+result.syncOK);
		        //console.dir(result);
		        if (result.syncOK === true) {
		        	console.log('syncableDatabase syncNow done OK');
		       	}else{
		       		console.error('syncableDatabase syncNow failed');
		       	}
		   },
	       true
	       );
	   	});

    webSqlDb.transaction(function(tx) {
      tx.executeSql('CREATE TABLE IF NOT EXISTS preferences (name varchar, value varchar)');
      tx.executeSql('CREATE TABLE IF NOT EXISTS testResults (id integer primary key, date datetime, baseline integer, current integer, score integer, within48hours bit, type varchar, urineRate float, urineHours integer)');
      //match CalcFurtherController.risks
      tx.executeSql('CREATE TABLE IF NOT EXISTS testResultsFurther (id integer primary key, date datetime, testResults_id integer, emailRequested bit, emailDone bit, emailTo varchar, age integer, baseline integer, heart integer,diabetes integer, nephrotoxic integer, contrast integer, sepsis integer, previousAKI integer, dementia integer, liver bit, hypovolaemia bit, newonset bit, earlywarning bit )');
      //match CalcRiskAccessController.risks
      tx.executeSql('CREATE TABLE IF NOT EXISTS riskAssesment(id integer primary key, date datetime, testResults_id integer, pot float, ph float, confusion bit, flap bit, rub bit, oedema bit)');
      //match CalcTreatmentController.steps
      tx.executeSql('CREATE TABLE IF NOT EXISTS treatment(id integer primary key, date datetime, testResults_id integer, fluid bit, chart bit, urine bit, ultrasound bit, catheter bit, stop bit, doses bit, lmwh bit, bloods bit)');
      //match ReferController.criteria
      tx.executeSql('CREATE TABLE IF NOT EXISTS refer (id integer primary key, date datetime, testResults_id integer, aki integer, ckd integer, transplant integer, haematuria integer, proteinurea integer, nonresolving integer, noclear integer)');
      //match ReferSafetyController.steps
      tx.executeSql('CREATE TABLE IF NOT EXISTS referSafety (id integer primary key, date datetime, testResults_id integer, refer_id integer, isSafe bit, pot int, ph int, bicarbonate int, rate int, sat int, insp int, cpap bit, pulse int, press int, level varchar, supp bit, vaso bit, copd bit)');

      tx.executeSql('CREATE TABLE IF NOT EXISTS referEmail (id integer primary key, date datetime, referSafety_id integer,emailDone bit, emailTo varchar, refer_id integer)',null,null,function(tx,e){console.error('syncableDatabase CREATE TABLE IF NOT EXISTS referEmail !ok'+e.message)} );

      tx.executeSql('CREATE TABLE IF NOT EXISTS reg (id integer primary key, date datetime, name varchar,email varchar,role varchar,hospital varchar, roleOther varchar, hospitalOther varchar)',null,null,function(tx,e){console.error('syncableDatabase CREATE TABLE IF NOT EXISTS reg error'+e.message)} );

      tx.executeSql('CREATE TABLE IF NOT EXISTS feedback2 (id integer primary key, date datetime, name varchar, email varchar, feedback varchar )',null,null,function(tx,e){} );

      tx.executeSql('ALTER TABLE testResults add column isTest bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE testResults add column type varchar',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE testResults add column urineRate float',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE testResults add column urineHours integer',null,null,function(tx,e){});

      tx.executeSql('ALTER TABLE refer add column isTest bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE referEmail add column region varchar',null,null,function(tx,e){});

      tx.executeSql('ALTER TABLE referSafety add column copd bit',null,null,function(tx,e){});

      tx.executeSql('ALTER TABLE treatment add column cause bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column reqBloods bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column dipstick bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column fluidStatus bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column management bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column balance bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column reqUltrasound bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column urineCatheter bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column nephrotoxic bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column reviewDoses bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE treatment add column disease bit',null,null,function(tx,e){});

      tx.executeSql('ALTER TABLE testResultsFurther add column newonset bit',null,null,function(tx,e){});
      tx.executeSql('ALTER TABLE testResultsFurther add column earlywarning bit',null,null,function(tx,e){});
    }
	,
    function(e){
    	console.error('failed to CREATE');
    	console.dir(e);
    });

	return {
		init:function(platform,id){
		   window.DBSYNC.log=function(){};
		   window.DBSYNC.initSync(
		      [
			      {tableName : 'testResults'},
			      {tableName : 'testResultsFurther'},
			      {tableName : 'riskAssesment'},
			      {tableName : 'treatment'},
			      {tableName : 'refer'},
			      {tableName : 'referSafety'},
			      {tableName : 'referEmail'},
			      {tableName : 'reg'},
			      {tableName : 'feedback2'}
			  ],
		      webSqlDb,
		      {deviceId:id,platform:platform,brand:'core'},
		      'https://app.akicare.co.uk/sync.cfml',
		      function callBackEndInit(){
		      	//console.log('initSync callBackEndInit')
		      },
		      null,
		      null,
		      60*1000
		    );
		},
	   	db: function(){ return webSqlDb; },
		//https://gist.github.com/jgoux/10738978
	   	executeSql:function(query, bindings) {
			var deferred = $q.defer();

	        bindings = typeof bindings !== 'undefined' ? bindings : [];
	        var deferred = $q.defer();

	        self.db.transaction(function(transaction) {
	            transaction.executeSql(query, bindings, function(transaction, result) {
	                deferred.resolve(result);
	                if( query.toLowerCase().indexOf('select')!=0){ self.sync(); }
	            }, function(transaction, error) {
	            	if(console&&console.dir) console.dir(error);
	                deferred.reject(error);
	            });
	        });

	        return deferred.promise;
	    },
	   	sync: self.sync,

	   	/* utils */

	    resultRowToStruct: function ( rowObj ){
	    	var st={};
	    	for (var col in rowObj){
	    		st[col]=rowObj[col];
	    	}
	    	return st;
	    }
	}
});
