"use strict";
angular.module('email-ask.service', [])
.factory('createEmailDialog', ["$document", "$rootScope", "$ionicPopover","syncableDatabase",'registerService',
function ($document,$rootScope,$ionicPopover,$syncService, registerService) {
  var defaults = {
    success: null,
    cancel:  null,
    scope: null, // must implement same funcs we save to scope below
  };
  var body = $document.find('body');

  return function Dialog(evt/*optional*/, options, passedInLocals) {

    options = angular.extend({}, defaults, options); //options defined in constructor

    var _preferenceKey="last_sent_to_email";
    var scope = options.scope || $rootScope.$new();

    scope.$hasSavedValue=false;
    scope.popoverEmail='';

    scope.$on('$destroy', function() {
      scope.popover.remove();
    });

    var closeFn = function () {
        scope.popover.remove()
     };
    scope.$modalClose = closeFn;

    scope.$modalCancel = function () {
      var callFn = options.cancel || closeFn;
      callFn.call(this);
      scope.$modalClose();
    };

    scope.$modalOk = function (form){
      if( ! form.inputPopoverEmail.$valid ){
        return;
      }

      var email=form.inputPopoverEmail.$modelValue;

      $syncService.db().transaction(function(tx){
        if( scope.$hasSavedValue ){
          tx.executeSql('update preferences set value=? where name=?',[email,_preferenceKey]);
        }else{
          tx.executeSql('insert into preferences (value,name) values (?,?)',[email,_preferenceKey]);
        }
      });

      var callFn = options.success || closeFn;
      callFn.call(this,email);
      scope.$modalClose();
    };

    $ionicPopover.fromTemplateUrl('templates/popover-email.html', {
      scope: scope,
      focusFirstInput: true
    }).then(function(popover){
      scope.popover=popover;

      $syncService.db().transaction(function(tx){
        tx.executeSql('select value from preferences where name=?',[_preferenceKey], function (tx,results){
          if( results.rows.length == 1 ){
            scope.$hasSavedValue=true;
            scope.popoverEmail=results.rows.item(0).value;
            scope.popover.show(evt);
          }else{
            if( registerService.registered() ){
            scope.popoverEmail=registerService.get().email;
            scope.popover.show(evt);
            }
            scope.popover.show(evt);
          }
        },
        function (){
            scope.popover.show(evt);
        });
      },
      function (){
          scope.popover.show(evt);
      });
    });

  };
}]);
